var path = require('path');
var { Tray, Menu } = require('electron').remote;
var { ipcRenderer, remote, shell } = require('electron');
const mainProcess = remote.require('./main');

const MAXIMUM_NUMBER_COUNT = 7;

const PREMIUM_USER_THRESHOLD = 1099511627776;

const INCOMING_SHARES = 1;
const OUTGOING_SHARES = 0;

const LINK_INITIAL_ADDRESS_DOWNLOAD = "https://my.pcloud.com/publink/show?code=";
const LINK_INITIAL_ADDRESS_UPLOAD = "https://my.pcloud.com/#page=puplink&code=";

const HIDE_LINK_ICON = 0;
const SHOW_INACTIVE_LINK_ICON = 1;
const SHOW_ACTIVE_LINK_ICON = 2;

const SHARES_FONTCOLOR_GRAY = 0;
const SHARES_FONTCOLOR_NORMAL = 1;

const INVITE_STYLE_HIDE = 0;
const INVITE_STYLE_GRAY = 1;
const INVITE_STYLE_ACTIVE = 2;

const WANTED_TYPE_FILES = 0;
const WANTET_TYPE_FOLDERS = 1;
const WANTED_TYPE_VIRTUALFOLDERS = 2;

const INCOMING_SHARES_ICON_ENVELOPE = 0;
const INCOMING_SHARES_ICON_BUSINESS = 1;

const INCOMING_SHARES_PERMISSION_CANVIEW = 0;
const INCOMING_SHARES_PERMISSION_CANEDIT = 1;
const INCOMING_SHARES_PERMISSION_CANMANAGE = 2;

const INCOMING_SHARES_BUTTON_STYLE_DISABLE = 0;
const INCOMING_SHARES_BUTTON_STYLE_ENABLE = 1;

const FOLDER_ICON_DEFAULT_ID = 20;

const oneHourInMilisecs = 3600000;
const sixHoursInMilisecs = 21600000;
const dayInMilisecs = 86400000;
const oneWeekInMilisecs = 604800000;
const neverRemindForUpdate = 1;

var extScope = null;
var route = null;

//style the options to make them look like tabs
var active = document.getElementById('shares');
active.style.borderBottom = '2px solid black';

var downloadStatus = document.getElementById('downloadStatus');
var uploadStatus = document.getElementById('uploadStatus');

var setActive = function (id) {
    var current = document.getElementById(id);
    if (active !== current) {
        if (id === 'shares') {
            startSpinner();
        }
        active.style.borderBottom = '0';
        current.style.borderBottom = '2px solid black';
        active = current;
    }
};

// Prevent opening files from drag and drop
document.addEventListener('dragover', function (event) {
    event.preventDefault();
    return false;
}, false);

document.addEventListener('drop', function (event) {
    event.preventDefault();
    return false;
}, false);

//main module for the app
var cloudApp = angular.module('AppModule', ['ngRoute', 'ngMaterial', 'ngMessages', 'ngAnimate', 'ngAria', 'material.svgAssetsCache'])
    //controller for the Sync option
    .controller('SyncController', function ($route) {
        var vm = this;
        route = $route;
        mainProcess.setCurrentOpenTab("sync");
        vm.folderlist = mainProcess.getSyncList();

        //Open the sync window
        vm.openAddSyncWindow = function () {
            mainProcess.addNewSyncWindow();
        };

        //Delete sync and refresh
        vm.deleteSync = function (syncid) {
            if (confirm('Are you sure you want to delete this sync?')) {
                ipcRenderer.send('delete-sync', syncid);
                route.reload();
            }
        };
    })

    //controller for the crypto option
    .controller('CryptoController', function ($scope, $route) {

        var vm = this;
        extScope = $scope;
        route = $route;
        mainProcess.setCurrentOpenTab("crypto");
        vm.cryptoStatus = mainProcess.checkCryptoStatus();
        resetCryptoForm();
        vm.showGracePeriod = mainProcess.cryptoExpiredGracePeriod();

        vm.openSetupPassword = function () {
            vm.cryptoStatus = 2;
        }

        vm.createCryptoFolder = function (cryptoPass, repeateCryptoPass, hint) {
            if (vm.passValueStr > 3) {
                if (cryptoPass === repeateCryptoPass) {
                    if (hintNotExistInCryptoPass(cryptoPass, hint)) {
                        let response = mainProcess.cryptoPassMemoryConfirmation();
                        if (response) {
                            let result = mainProcess.cryptoSetup(cryptoPass, hint);
                            if (result) {
                                initiateLoading(2000);
                                vm.cryptoStatus = 3;
                            }
                        } else {
                            resetCryptoForm();
                        }
                    } else {
                        mainProcess.showError("Hint must not contain the Crypto Pass!")
                    }
                } else {
                    mainProcess.showError("The two Crypto Passes should match.")
                }

            } else {
                mainProcess.showError("The Crypto Pass is too weak. You need to provide a sufficient Crypto Pass.");
            }
        }

        function hintNotExistInCryptoPass(cryptoPass, hint) {
            if (hint === undefined || hint === "") {
                return true;
            }
            if (hint.indexOf(cryptoPass) === -1) {
                return true;
            } else {
                return false;
            }
        }

        function resetCryptoForm() {
            var spanText = document.getElementById('crypto-pass-string');
            vm.cryptoPass = "";
            vm.repeateCryptoPass = "";
            vm.cryptoStrengthString = "None";
            spanText.style.color = "#4E4E4E";
            vm.passValueStr = 0;
            vm.cryptoPassMatch = false;
        }

        vm.cryptofieldpassKeypress = function () {
            vm.passValueStr = mainProcess.cryptoPassDifficulty10000(vm.cryptoPass);
            vm.cryptoStrengthString = setCryptoPassStrengthString(vm.passValueStr);
            vm.checkIfCryptoPassMatches();
        }

        vm.checkIfCryptoPassMatches = function () {
            if (vm.cryptoPass === vm.repeateCryptoPass && vm.cryptoPass !== "" && vm.cryptoPass !== undefined) {
                vm.cryptoPassMatch = true;
            } else {
                vm.cryptoPassMatch = false;
            }
        }

        function setCryptoPassStrengthString(passStr) {
            var spanText = document.getElementById('crypto-pass-string');

            if (passStr === 0) {
                spanText.style.color = "#4E4E4E";
                return "None";
            } else if (passStr <= 3) {
                spanText.style.color = "#FF4D4D";
                return "Weak";
            } else if (passStr <= 6) {
                spanText.style.color = "#FF9326";
                return "Medium";
            } else {
                spanText.style.color = "#83C100";
                return "Strong";
            }
        }

        // https://www.pcloud.com/bg/cloud-storage-pricing-plans.html?product=crypto&authtoken=

        vm.subscribeCrypto = function () {
            let token = mainProcess.getAuthToken();
            shell.openExternal("https://www.pcloud.com/bg/cloud-storage-pricing-plans.html?product=crypto&authtoken=" + token);
        }

        vm.cryptoUnlocked = mainProcess.cryptoStarted();

        vm.unlockCrypto = function () {
            ipcRenderer.send('unlock-crypto-window');
        }

        vm.lockCrypto = function () {
            initiateLoading(1000);
            let result = mainProcess.lockCryptoFolder();
            if (result) {
                vm.cryptoUnlocked = false;
            }
        }
        if (vm.cryptoStatus === 3 || vm.cryptoStatus === 4) {
            vm.cryptoExpirationDays = mainProcess.getCryptoExpirationDaysString();
        } else if (vm.cryptoStatus === 1 && mainProcess.cryptoExpiredGracePeriod()) {
            vm.cryptoExpirationDays = "Crypto Expired!"
        }
    })

    /*Settings keys are:
    upSpeed ("unlimited", "auto", value (int)) - maximum upload speed in bytes per second, 0 for auto-shaper, -1 for no limit
    dwSpeed ("unlimited", "auto", value (int)) - maximum download speed in bytes per second, 0 for auto-shaper, -1 for no limit
    lanSync (boolean) - use or not peer to peer downloads
    sleepstopcrypto (bool) - if set, stops crypto when computer wakes up from sleep
    minDiskSpace (uint) - minimum free space on local drives to run downloads
    cacheSize (uint) - size of filesystem cache, in bytes, sane minimum of few tens of Mb or even hundreds is advised
    autostart(boolean)
    */
    .controller('SettingsController', [function () {
        var vm = this;
        route = null;
        vm.loading = false;
        mainProcess.setCurrentOpenTab("settings");
        vm.showCacheWarning = false;
        // vm.showCryptoInSettings = false;

        // CRYPTO SETTINGS
        vm.showCryptoInSettings = showCryptoSettings();

        vm.unlockCryptoOnStart = mainProcess.getBoolValue("unlockcryptoonstart");

        vm.resetCrypto = function () {
            let confirmation = mainProcess.resetCryptoPassConfirmation();
            if (confirmation) {
                mainProcess.resetCryptoPass();
            }
        }

        //DISABLE THE BUTTONS
        vm.activateButtons = true;

        // GET THE CURRENT SETTINGS
        var initSettings = mainProcess.getAllSettings();
        vm.currentSettings = Object.assign({}, initSettings);

        // CHOOSE THE CURRENT RADIO BUTTTON ACCORDING TO THE DB SETTING
        vm.dwSpeedRadioChoice = radioChoiceConverter(vm.currentSettings.dwSpeed);
        vm.upSpeedRadioChoice = radioChoiceConverter(vm.currentSettings.upSpeed);
        vm.minDiskSpace = bytesToMBConverter(vm.currentSettings.minDiskSpace);
        vm.cacheSize = bytesToMBConverter(vm.currentSettings.cacheSize);

        // IF THE SETTING IS ON CUSTOM DO NOT WIPE-OUT THE VALUE INTO THE FIELD
        if (vm.upSpeedRadioChoice === 3) {
            vm.custUpSpeedValue = vm.currentSettings.upSpeed;
        }
        if (vm.dwSpeedRadioChoice === 3) {
            vm.custDwSpeedValue = vm.currentSettings.dwSpeed;
        }

        // HELP VARIABLE TO CHECK FOR DIFFERENCE EASIER
        var dwSpeedRadioChoiceTemp = vm.dwSpeedRadioChoice;
        var upSpeedRadioChoiceTemp = vm.upSpeedRadioChoice;

        //UPDATE THE NEW SETTINGS INTO THE DB
        vm.apply = function () {

            initiateLoading(1500);

            if (vm.dwSpeedRadioChoice === "3") {

                if (vm.custDwSpeedValue !== vm.currentSettings.dwSpeed && vm.custDwSpeedValue !== null && vm.custDwSpeedValue !== undefined && vm.custDwSpeedValue !== 0) {
                    vm.currentSettings.dwSpeed = vm.custDwSpeedValue;

                } else if (vm.currentSettings.dwSpeed === 0) {
                    vm.dwSpeedRadioChoice = "2";
                    dwSpeedRadioChoiceTemp = vm.dwSpeedRadioChoice;
                    vm.custDwSpeedValue = null;
                }
            } else {
                vm.custDwSpeedValue = null;
            }

            if (vm.upSpeedRadioChoice === "3") {
                if (vm.custUpSpeedValue !== vm.currentSettings.upSpeed && vm.custUpSpeedValue !== null && vm.custUpSpeedValue !== undefined && vm.custUpSpeedValue !== 0) {
                    vm.currentSettings.upSpeed = vm.custUpSpeedValue;

                } else if (vm.currentSettings.upSpeed === 0) {
                    vm.upSpeedRadioChoice = "2";
                    upSpeedRadioChoiceTemp = vm.upSpeedRadioChoice;
                    vm.custUpSpeedValue = null;
                }
            } else {
                vm.custUpSpeedValue = null;
            }

            var arrayWithChanges = {};

            for (var key in vm.currentSettings) {

                if (vm.currentSettings[key] !== initSettings[key]) {
                    arrayWithChanges[key] = vm.currentSettings[key];
                }
            }

            // console.log("arrayWithChanges.length -> " + Object.keys(arrayWithChanges).length);

            if (Object.keys(arrayWithChanges).length > 0) {
                ipcRenderer.send('update-settings', arrayWithChanges);
                initSettings = Object.assign({}, vm.currentSettings);
            }

            vm.activateButtons = true;
            dwSpeedRadioChoiceTemp = vm.dwSpeedRadioChoice;
            upSpeedRadioChoiceTemp = vm.upSpeedRadioChoice;

            if (vm.unlockCryptoOnStart != mainProcess.getBoolValue("unlockcryptoonstart")) {
                if (vm.unlockCryptoOnStart === false) {
                    mainProcess.setBoolValue("unlockcryptoonstart", 0);
                } else {
                    mainProcess.setBoolValue("unlockcryptoonstart", 1);
                }
            }
        }

        // RETURN THE SETTINGS INTO INITIAL STATE (CANCEL BUTTON CLICKED)
        vm.restoreInitialSettings = function () {
            vm.currentSettings = Object.assign({}, initSettings);
            vm.showCacheWarning = false;
            vm.dwSpeedRadioChoice = radioChoiceConverter(initSettings.dwSpeed);
            vm.upSpeedRadioChoice = radioChoiceConverter(initSettings.upSpeed);
            vm.minDiskSpace = bytesToMBConverter(initSettings.minDiskSpace);
            vm.cacheSize = bytesToMBConverter(initSettings.cacheSize);
            dwSpeedRadioChoiceTemp = vm.dwSpeedRadioChoice;
            upSpeedRadioChoiceTemp = vm.upSpeedRadioChoice;
            if (vm.dwSpeedRadioChoice === 1 || vm.dwSpeedRadioChoice === 2) {
                vm.custDwSpeedValue = null;
            } else {
                vm.custDwSpeedValue = initSettings.dwSpeed;
            }

            if (vm.upSpeedRadioChoice === 1 || vm.upSpeedRadioChoice === 2) {
                vm.custUpSpeedValue = null;
            } else {
                vm.custUpSpeedValue = initSettings.upSpeed;
            }

            vm.unlockCryptoOnStart = mainProcess.getBoolValue("unlockcryptoonstart");
            vm.activateButtons = true;
        }

        // USED TO ACTIVATE THE "APPLY" AND "CANCEL" BUTTONS. Also Apply the new values to the variables OF THE CURRENT SETTINGS (DO NOT APPLY THE SETTINGS INTO THE DB)
        vm.initiatedActivityOnSettings = function () {

            let shouldActivate = false;

            let tempMinDiskSpaceValueInBytes = megaBytesToBytesConverter(vm.minDiskSpace);
            let tempCacheSizeValueInBytes = megaBytesToBytesConverter(vm.cacheSize);

            // COMPARE THE CURRENT VALUES WITH THE ORIGINAL OF DISK USAGE TAB
            if (tempMinDiskSpaceValueInBytes !== vm.currentSettings.minDiskSpace) {
                vm.currentSettings.minDiskSpace = tempMinDiskSpaceValueInBytes;
            }
            if (vm.cacheSize < 256) {
                vm.showCacheWarning = true;
                vm.activateButtons = true;
            } else {
                if (tempCacheSizeValueInBytes !== vm.currentSettings.cacheSize) {
                    vm.showCacheWarning = false;
                    vm.activateButtons = false;
                    vm.currentSettings.cacheSize = tempCacheSizeValueInBytes;
                }
            }

            // COMPARE THE VALUES OF SPEED TAB
            if (vm.upSpeedRadioChoice !== upSpeedRadioChoiceTemp) {

                upSpeedRadioChoiceTemp = vm.upSpeedRadioChoice;

                if (vm.upSpeedRadioChoice === "1") {
                    vm.currentSettings.upSpeed = -1;

                } else if (vm.upSpeedRadioChoice === "2") {

                    vm.currentSettings.upSpeed = 0;
                } else {

                    if (vm.custUpSpeedValue === 0 || vm.custUpSpeedValue === null || vm.custUpSpeedValue === undefined) {
                        vm.currentSettings.upSpeed = 0;
                        shouldActivate = true;

                    } else {
                        vm.currentSettings.upSpeed = vm.custUpSpeedValue;
                    }
                }
            } else {

                if (vm.upSpeedRadioChoice === 1 || vm.upSpeedRadioChoice === 2) {
                    vm.custUpSpeedValue = null;

                } else {
                    if (vm.custUpSpeedValue !== vm.currentSettings.upSpeed) {
                        if (vm.custUpSpeedValue === 0 || vm.custUpSpeedValue === null || vm.custUpSpeedValue === undefined) {
                            vm.currentSettings.upSpeed = 0;
                        } else {
                            vm.currentSettings.upSpeed = vm.custUpSpeedValue;
                        }
                    }
                }
            }

            if (vm.dwSpeedRadioChoice !== dwSpeedRadioChoiceTemp) {

                dwSpeedRadioChoiceTemp = vm.dwSpeedRadioChoice;

                if (vm.dwSpeedRadioChoice === "1") {
                    vm.currentSettings.dwSpeed = -1;

                } else if (vm.dwSpeedRadioChoice === "2") {
                    vm.currentSettings.dwSpeed = 0;

                } else {

                    if (vm.custDwSpeedValue === 0 || vm.custDwSpeedValue === null || vm.custDwSpeedValue === undefined) {
                        shouldActivate = true;
                        vm.currentSettings.dwSpeed = 0;

                    } else {
                        vm.currentSettings.dwSpeed = vm.custDwSpeedValue;
                    }
                }

            } else {

                if (vm.dwSpeedRadioChoice === 1 || vm.dwSpeedRadioChoice === 2) {
                    vm.custDwSpeedValue = null;

                } else {

                    if (vm.custDwSpeedValue !== vm.currentSettings.dwSpeed) {

                        if (vm.custDwSpeedValue === 0 || vm.custDwSpeedValue === null || vm.custDwSpeedValue === undefined) {
                            vm.currentSettings.dwSpeed = 0;
                        } else {
                            vm.currentSettings.dwSpeed = vm.custDwSpeedValue;
                        }
                    }
                }
            }

            if (vm.unlockCryptoOnStart != mainProcess.getBoolValue("unlockcryptoonstart")) {
                shouldActivate = true;
            }

            // COMPARE ALL CURRENT VALUES WITH THE ORIGINAL

            if (shouldActivate) {
                vm.activateButtons = false;
            } else if (!vm.showCacheWarning) {
                let initialStateRequired = JSON.stringify(initSettings) === JSON.stringify(vm.currentSettings)

                if (initialStateRequired) {
                    vm.activateButtons = true;
                } else {
                    vm.activateButtons = false;
                }
            }
        }

        // Function to prevent the user input of more than 8 digits
        vm.customValueKeyPressed = function (event, value, valueType) {
            let keyCode = ('which' in event) ? event.which : event.keyCode;
            if (value !== undefined && value.toString().length > MAXIMUM_NUMBER_COUNT || !FilterInput(event)) {
                event.preventDefault();
            } else if ((value === null && keyCode === 48) || (value === 0 && keyCode === 48)) {
                event.preventDefault();
            } else if (value !== undefined) {
                if (valueType === "upload") {
                    vm.currentSettings.upSpeed = vm.custUpSpeedValue;
                    vm.initiatedActivityOnSettings();
                } else if (valueType === "download") {
                    vm.currentSettings.dwSpeed = vm.custDwSpeedValue;
                    vm.initiatedActivityOnSettings();
                } else if (valueType === "cacheSize") {
                    vm.cacheSize = value;
                }
            }
        }

        vm.handlePaste = function (e) {

            var clipboardData, pastedData;

            // Get pasted data via clipboard API
            clipboardData = e.clipboardData || window.clipboardData;
            pastedData = clipboardData.getData('Text').toUpperCase();
            // console.log(clipboardData);
            let pasteFieldThreshold = pastedData.length + e.srcElement.value.length;
            if (pastedData.indexOf('E') > -1 || pastedData.indexOf('-') > -1 || pastedData.indexOf('+') > -1 || pastedData.indexOf('.') > -1 || pastedData.charAt(0) === '0' || pasteFieldThreshold > (MAXIMUM_NUMBER_COUNT + 1)) {
                e.stopPropagation();
                e.preventDefault();
            }
        };

        // THESE 4 ARE USING TO CHANGE THE FOCUS TO THE INPUT FIELD
        vm.changeFocusToUpForm = function () {
            document.getElementById("upCustomInput").focus();
        }
        vm.changeFocusToDwForm = function () {
            document.getElementById("dwCustomInput").focus();
        }
        vm.focusUploadInputAndChangeRadioChoice = function () {
            setTimeout(
                function () {
                    document.getElementById("upCustomInput").focus();
                }, 50);
            vm.upSpeedRadioChoice = '3';
        }
        vm.focusDownloadInputAndChangeRadioChoice = function () {
            setTimeout(
                function () {
                    document.getElementById("dwCustomInput").focus();
                }, 50);
            vm.dwSpeedRadioChoice = '3';
        }

        // SETTING TABZZZ UI
        vm.activeTab = 2;

        vm.cacheLocation = mainProcess.getCurrentCacheLoc();

        vm.clearCache = function () {
            mainProcess.clearCache();
            initiateLoading(3000);
        }

        vm.changeCacheLocation = function () {
            mainProcess.changeCacheLocation();
            vm.cacheLocation = mainProcess.getCurrentCacheLoc();
        }
    }])

    // Controller for the account tab
    .controller('AccountController', function ($scope, $http) {

        var vm = this;
        extScope = $scope;
        route = null;
        mainProcess.setCurrentOpenTab("account");
        var openDriveElement = document.getElementById('account-open-drive');
        openDriveElement.addEventListener('click', function (event) {
            shell.showItemInFolder(process.env['HOME'] + '/pCloudDrive/./');
        });

        var openMypCloud = document.getElementById('account-open-mypcloud');
        openMypCloud.addEventListener('click', function (event) {
            let token = mainProcess.getAuthToken();
            shell.openExternal('https://my.pcloud.com/#page=filemanager&authtoken=' + token);
        });

        var openTrashElement = document.getElementById('account-open-trash');
        openTrashElement.addEventListener('click', function (event) {
            let token = mainProcess.getAuthToken();
            shell.openExternal('https://my.pcloud.com/#page=trash&authtoken=' + token);

        });

        var moreSpaceButton = document.getElementById('more-space');
        moreSpaceButton.addEventListener('click', function (event) {
            let token = mainProcess.getAuthToken();
            shell.openExternal('https://my.pcloud.com/#page=space&authtoken=' + token);
        });

        var upgradeToPremiumButton = document.getElementById('upgrade-premium');
        upgradeToPremiumButton.addEventListener('click', function (event) {
            let token = mainProcess.getAuthToken();

            shell.openExternal('https://www.pcloud.com/cloud-storage-pricing-plans.html?authtoken=' + token);
        });

        var logoutButton = document.getElementById('logout-button');
        logoutButton.addEventListener('click', function (event) {
            ipcRenderer.send('logout-pressed');
        });

        var unlink = document.getElementById('unlink-button');
        unlink.addEventListener('click', function (event) {
            ipcRenderer.send('unlink-pressed');
        });

        vm.openMyAccount = function () {
            let token = mainProcess.getAuthToken();
            shell.openExternal('https://my.pcloud.com/#page=b_account&tab-baccount=myaccount&authtoken=' + token);
        }

        // let token = mainProcess.getAuthToken();

        // var planCheckerRequest = 'https://api.pcloud.com/userinfo?auth=' + token;
        // var userPlan = null;
        // $http.get(planCheckerRequest)
        //     .then(function (response) {
        //         userPlan = response.data.plan;
        //     });

        var totalQuotaRaw = mainProcess.getTotalQuota();
        if (totalQuotaRaw >= PREMIUM_USER_THRESHOLD) {
            upgradeToPremiumButton.disabled = true;
            moreSpaceButton.disabled = true;
        } else {
            upgradeToPremiumButton.disabled = false;
            moreSpaceButton.disabled = false;
        }

        var userOnlyDiv = document.getElementById('onlyUserInfo');
        var businessDiv = document.getElementById('business-info');
        userOnlyDiv.style.visibility = "hidden";
        businessDiv.style.visibility = "hidden";

        if (mainProcess.isUserBusiness()) {
            vm.companyName = mainProcess.getCompanyName();
            userOnlyDiv.style.visibility = "hidden";
            var businessDiv = document.getElementById('business-info');
            businessDiv.style.visibility = "visible";
            // upgradeToPremiumButton.disabled = true;
            // moreSpaceButton.disabled = true;
        } else {
            businessDiv.style.visibility = "hidden";
            var userOnlyDiv = document.getElementById('onlyUserInfo');
            userOnlyDiv.style.visibility = "visible";
            // upgradeToPremiumButton.disabled = false;
            // moreSpaceButton.disabled = false;
        }
        vm.verified = mainProcess.getIfUserNeedVerification();

        vm.verify = function () {
            ipcRenderer.send('verify-email');
        }

        vm.changePassword = function () {
            ipcRenderer.send('change-password');
        }

        vm.forgotPassword = function () {
            // shell.openExternal('https://my.pcloud.com/#page=login');
            ipcRenderer.send('forgot-password');
        }

        vm.username = mainProcess.getUserName();
        var usedQuotaRaw = mainProcess.getUsedQuota();

        vm.usedQuota = quotaConverter(usedQuotaRaw, 2);
        vm.totalQuota = quotaConverter(totalQuotaRaw, 2);
        vm.progressValue = getProgressValue(usedQuotaRaw, totalQuotaRaw);
    })

    .controller('SharesController', function ($route, $scope) {
        var vm = this;
        route = $route;
        extScope = $scope;
        vm.activeTab = 1;
        mainProcess.setCurrentOpenTab("share");
        vm.inviteToFolderTooltip = "Invite to Folder";
        vm.createUploadLink = "Copy Upload Link";
        vm.createDownloadLink = "Copy Download Link";
        vm.openFolderTooltip = "Double-click to Open";
        vm.shareButtonVisible = false;
        vm.selectedDrpIndex = -1;

        // Gather all the outgoing shares from the different types
        let outgoingSharesList = mainProcess.getShareList(OUTGOING_SHARES);
        let outgoingShareRequestsList = mainProcess.getShareRequestList(OUTGOING_SHARES);
        let publicLinks = mainProcess.getPublicLinks();
        // All outgoing shares
        var allOutgoingSharesRaw = outgoingSharesList.concat(outgoingShareRequestsList, publicLinks);

        // Gather all the outgoing shares from the different types
        let incomingSharesList = mainProcess.getShareList(INCOMING_SHARES);
        let incomingShareRequestsList = mainProcess.getShareRequestList(INCOMING_SHARES);

        // All incoming shares

        // Get all contacts
        let contactsArray = mainProcess.getContactsList();

        // var allIncomingSharesRaw = incomingSharesList.concat(incomingShareRequestsList);

        // Gather all outgoing shares into specific models devided by type (files, folders, virtual folders)
        let allOutgoingFoldersObject = getMeOutgoingSharesModel(WANTET_TYPE_FOLDERS, allOutgoingSharesRaw);
        let allOutgoingVirtualFoldersObject = getMeOutgoingSharesModel(WANTED_TYPE_VIRTUALFOLDERS, allOutgoingSharesRaw);
        let allOutgoingFilesObject = getMeOutgoingSharesModel(WANTED_TYPE_FILES, allOutgoingSharesRaw);

        // Gather all incoming shares into specific models devided by type (pending, accepted)
        let allIncomingAcceptedSharesArray = getMeIncomingSharesModel('accepted', incomingSharesList);
        let allIncomingPendingSharesArray = getMeIncomingSharesModel('pending', incomingShareRequestsList);

        // Setting the icon property of each virtual folder so that it can be displayed differently from normal folders
        allOutgoingVirtualFoldersObject.forEach(function (share) {
            share.icon = 666;
        });

        // Sorting all arrays Lexicographically
        allOutgoingFoldersObject.sort(sortLexicographically);
        allOutgoingVirtualFoldersObject.sort(sortLexicographically);
        allOutgoingFilesObject.sort(sortLexicographically);

        vm.allOutgoingShares = allOutgoingFoldersObject.concat(allOutgoingVirtualFoldersObject, allOutgoingFilesObject);

        vm.allPendingInvitations = allIncomingPendingSharesArray.sort(sortLexicographically);
        vm.allAcceptedShares = allIncomingAcceptedSharesArray.sort(sortLexicographically);

        contactsNameFilter(vm.allAcceptedShares, contactsArray);

        vm.openDirectory = function (share) {
            share.openAction();
        }

        vm.stopAllShares = function (stopSharesArray) {
            initiateLoading(2000);
            let confirmation = mainProcess.stopShareConfirmation('my-shares');
            if (confirmation) {
                stopSharesArray.forEach(function (shareCancel) {
                    shareCancel();
                });
                ipcRenderer.send('refresh-appWindowShares-openSpecificTab', 1);
            }
        }

        vm.cancelAcceptedShareForFolder = function (shareAction, share) {
            console.log(share);
            if (shareAction.name === 'Cancel') {
                let confirmation = mainProcess.stopShareConfirmation('shared-with-me');
                if (confirmation) {
                    shareAction.action();
                }
                initiateLoading(2000);
            } else {
                shareAction.action();
            }
        }

        vm.invitationAction = function (action) {
            initiateLoading(2000);
            let result = action();
            if (result === 1) {
                ipcRenderer.send('refresh-appWindowShares-openSpecificTab', 2);
            }
        }

        vm.openChooseFolderToShareDialog = function () {
            mainProcess.openChooseFolderToShareDialog();
        }

        vm.permissionSetter = function (permission) {
            let response = "";
            switch (permission) {
                case 0:
                    response = "can view";
                    break;
                case 1:
                    response = "can edit";
                    break;
                case 2:
                    response = "can manage";
                    break;

                default:
                    response = 'unknown';
                    break;
            }
            return response;
        }

        vm.openUploadFolderPicker = function () {
            mainProcess.generateUploadLinkFromPicker();
        }

        vm.openDownloadFilePicker = function () {
            mainProcess.generateDownloadFileLinkFromPicker();
        }

        vm.openDownloadFolderPicker = function () {
            mainProcess.generateDownloadFolderLinkFromPicker();
        }

        vm.showDrpDownElement = function (index) {
            vm.selectedDrpIndex = index;
        }

        vm.closeDrpDownMenu = function () {
            vm.selectedDrpIndex = -1;
        }

        vm.showNewShareOptions = function () {
            vm.shareButtonVisible = true;
        }

        vm.hideNewShareOptions = function () {
            vm.shareButtonVisible = false;
        }

        // Declaring functions to determin the icons
        vm.iconPicker = iconLocator;
        vm.acceptSharesIconPicker = acceptSharesIconPicker;
        vm.iconUserPicker = iconUserPicker;
        vm.upLinkIconPicker = upLinkIconPicker;
        vm.dwLinkIconPicker = dwLinkIconPicker;
        stopSpinner();
    })

    .controller('HelpController', function () {
        var vm = this;
        route = null;
        mainProcess.setCurrentOpenTab("help");
    })

    .controller('AboutController', function ($scope, $route) {
        var vm = this;
        route = $route;
        extScope = $scope;
        vm.showPostponeChoiceContainer = false;

        vm.downloadingInProgress = mainProcess.isDownloadingInProgressFunc();
        // vm.downloadingInProgress = true;

        mainProcess.setCurrentOpenTab("about");
        // vm.updaterState = '3';
        vm.updaterState = getUpdaterState();

        // vm.newVersionReleased = mainProcess.newVersionChecker();

        // vm.showOnlyRestartOption = mainProcess.updateAlreadyDownloaded();


        // STATE-1 = New version AND ALREADY DOWNLOADED (NEED ONLY RESTART)
        // STATE-2 = New version AVAILABLE but not downloaded
        // STATE-3 = NO NEW VERSION

        function getUpdaterState() {
            let newVersion = mainProcess.newVersionChecker();
            let showOnlyRestartOption = mainProcess.updateAlreadyDownloaded();
            if (newVersion === true && showOnlyRestartOption === true) {
                return '1';
            } else if (newVersion === true && showOnlyRestartOption === false) {
                return '2';
            } else {
                return '3';
            }
        }

        vm.restart = function () {
            mainProcess.restartApplication();
        }

        vm.currentPostponeChoice = mainProcess.getUpdaterInterval();
        vm.releaseNotes = mainProcess.getReleaseNotes();
        vm.version = mainProcess.getUpdateVersion();

        vm.testFunction = function () {
            mainProcess.testFunction();
        }

        vm.showPostponeChoices = function () {
            vm.showPostponeChoiceContainer = true;
        }

        vm.hidePostponeChoices = function () {
            vm.showPostponeChoiceContainer = false;
        }

        vm.checkForUpdate = function () {
            mainProcess.manualCheckForUpdate();
        }

        vm.initiateUpdate = function () {
            mainProcess.initiateUpdate();
        }

        vm.setNewPostponeValue = function (postponeValue) {
            switch (postponeValue) {
                case '1hour':
                    vm.currentPostponeChoice = 'After One Hour';
                    mainProcess.setNextNotificationTime(oneHourInMilisecs);
                    vm.hidePostponeChoices();
                    break;
                case '6hours':
                    vm.currentPostponeChoice = 'After Six Hours';
                    mainProcess.setNextNotificationTime(sixHoursInMilisecs);
                    vm.hidePostponeChoices();
                    break;
                case '24hours':
                    vm.currentPostponeChoice = 'Tomorrow';
                    mainProcess.setNextNotificationTime(dayInMilisecs);
                    vm.hidePostponeChoices();
                    break;
                case '7days':
                    vm.currentPostponeChoice = 'Next Week';
                    mainProcess.setNextNotificationTime(oneWeekInMilisecs);
                    vm.hidePostponeChoices();
                    break;
                case 'never':
                    vm.currentPostponeChoice = 'Never';
                    mainProcess.setNextNotificationTime(neverRemindForUpdate);
                    vm.hidePostponeChoices();
                    break;
                default:
                    break;
            }
        }


    })

    //load different html files on option click
    .config(['$routeProvider', function ($routeProvider) {
        $routeProvider
            .when('/account', {
                templateUrl: 'partials/account.html',
                controller: 'AccountController as vm',
            })
            .when('/sync', {
                templateUrl: 'partials/sync.html',
                controller: 'SyncController as vm',
            })
            .when('/shares', {
                controller: 'SharesController as vm',
                templateUrl: 'partials/shares.html'

            })
            .when('/crypto', {
                templateUrl: 'partials/crypto.html',
                controller: 'CryptoController as vm'
            })
            .when('/settings', {
                templateUrl: 'partials/settings.html',
                controller: 'SettingsController as vm',
            })
            .when('/help', {
                templateUrl: 'partials/help.html',
                controller: 'HelpController as vm'
            })
            .when('/about', {
                templateUrl: 'partials/about.html',
                controller: 'AboutController as vm'
            })
            .otherwise({
                redirectTo: '/sync'
            });
    }]);

var showCryptoSettings = function () {
    // console.log("called showCryptoSettings")
    let cryptoStatus = mainProcess.checkCryptoStatus();
    if (cryptoStatus === 3 || cryptoStatus === 4) {
        return true;
    } else if (cryptoStatus === 1 && mainProcess.cryptoExpiredGracePeriod()) {
        return true;
    } else {
        return false;
    }
}

//refresh the page after a new sync is added
ipcRenderer.on('refresh-sync-page', function (event, args) {
    document.getElementById('shares').click();
    document.getElementById('sync').click();
});

function getProgressValue(usedQuota, totalQuota) {
    var progressValue = (usedQuota / totalQuota) * 100;
    return progressValue.toFixed(2);;
}

ipcRenderer.on('dw-status', (event, args) => {
    downloadStatus.innerHTML = args;

});

ipcRenderer.on('open-invitation-to-folder', (event, args) => {
    openInvitationToFolder(args[0], args[1], args[2], args[3]);
});

ipcRenderer.on('up-status', (event, args) => {
    uploadStatus.innerHTML = args;
});

ipcRenderer.on('initiateLoading', function () {
    initiateLoading(3000);
});

ipcRenderer.on('crypto-started', function () {
    initiateLoading(1000);
    if (extScope.vm.cryptoUnlocked != null && extScope.vm.cryptoUnlocked != undefined) {
        extScope.vm.cryptoUnlocked = mainProcess.cryptoStarted();
        route.reload();
    }
});

ipcRenderer.on('update-available', function () {
    if (extScope.vm.newVersionReleased !== mainProcess.newVersionChecker()) {
        // console.log("CALLED -> update-available at index.js")
        initiateLoading(2000);
        extScope.vm.releaseNotes = mainProcess.getReleaseNotes();
        extScope.vm.version = mainProcess.getUpdateVersion();
        extScope.vm.newVersionReleased = mainProcess.newVersionChecker();
        route.reload();
    }
});

ipcRenderer.on('show-only-restart-option', function () {
    initiateLoading(2000);
    extScope.vm.showOnlyRestartOption = '1';
    route.reload();
});

ipcRenderer.on('update-downloading', function () {
    extScope.vm.downloadingInProgress = true;
});

ipcRenderer.on('update-downloaded', function () {
    extScope.vm.downloadingInProgress = false;
});

ipcRenderer.on('set-active-tab', function (event, args) {
    if (extScope.vm.activeTab != null && extScope.vm.activeTab != undefined) {
        extScope.vm.activeTab = args;
    }
});

ipcRenderer.on('reload-shareRoute-page', function () {
    if (route !== null && route !== undefined) {
        route.reload();
    }
});

ipcRenderer.on('reload-share-page', function () {
    if (route !== null && route !== undefined) {
        if (extScope.vm.activeTab == 1) {
            route.reload();
        } else {
            ipcRenderer.send('refresh-appWindowShares-openSpecificTab', 2);
        }
    }
});

ipcRenderer.on('update-info', (event, args) => {
    let infoType = args[0];
    let value = args[1];

    if (extScope !== null) {
        extScope.$apply(function () {
            switch (infoType) {
                case 'newTotalQuota':
                    extScope.totalQuotaRaw = value;
                    extScope.vm.progressValue = getProgressValue(extScope.usedQuota, extScope.totalQuota);
                    extScope.vm.totalQuota = quotaConverter(extScope.totalQuotaRaw, 2);
                    break;
                case 'newUserUsedQuota':
                    extScope.usedQuotaRaw = value;
                    extScope.vm.progressValue = getProgressValue(extScope.usedQuota, extScope.totalQuota);
                    extScope.vm.usedQuota = quotaConverter(extScope.usedQuotaRaw, 2);
                    break;
                case 'newUserVerified':
                    extScope.vm.verified = value;
                    reloadScreen();
                    break;
                case 'newCryptoStatus':
                    extScope.vm.cryptoStatus = value;
                    if (route !== null && route !== undefined) {
                        route.reload();
                    }
                    break;
                case 'cryptoUnlockStatusChanged':
                    extScope.vm.cryptoUnlocked = value;
                    if (route !== null && route !== undefined) {
                        route.reload();
                    }
                    break;
                default:
                    break;
            }
        })
    }
});

var reloadScreen = function () {
    ipcRenderer.send('refresh-screen');
}

var radioChoiceConverter = function (dbSpeedValue) {
    if (dbSpeedValue === -1) {
        return 1;
    } else if (dbSpeedValue === 0) {
        return 2;
    } else {
        return 3;
    }
}

function quotaConverter(bytes, decimals) {
    if (bytes === 0) {
        return "0 Byte";
    }
    var k = 1024; //Or 1 kilo = 1000
    var sizes = ["Bytes", "KB", "MB", "GB", "TB", "PB"];
    var i = Math.floor(Math.log(bytes) / Math.log(k));

    // console.log('NUMBER IS - ' + parseFloat((bytes / Math.pow(k, i)).toFixed(decimals)) + " " + sizes[i]);

    return parseFloat((bytes / Math.pow(k, i)).toFixed(decimals)) + " " + sizes[i];

}

function bytesToMBConverter(bytes) {
    let sizeInMB = (bytes / (1024 * 1024)).toFixed(2);
    return Math.round(sizeInMB);
}

function megaBytesToBytesConverter(megabytes) {
    let sizeInBytes = (megabytes * (1024 * 1024)).toFixed(2);
    // console.log(Math.round(sizeInBytes));
    return Math.round(sizeInBytes);
}

function initiateLoading(spinningTime) {
    document.body.classList.add('loading');
    setTimeout(
        function () {
            document.body.classList.remove('loading');
        }, spinningTime);
}

function startSpinner() {
    document.body.classList.add('loading');
}

function stopSpinner() {
    document.body.classList.remove('loading');
}

function FilterInput(event) {
    let keyCode = ('which' in event) ? event.which : event.keyCode;
    isNotWanted = (keyCode === 69 || keyCode === 101 || keyCode === 45 || keyCode === 43 || keyCode === 46);
    return !isNotWanted;
};

function getMeOutgoingSharesModel(wantedType, sharesRaw) {

    var result = {};
    var justOutgoingShares = mainProcess.getShareList(OUTGOING_SHARES);

    for (var x = 0; x < sharesRaw.length; x++) {

        if (sharesRaw[x].isfolder !== undefined && sharesRaw[x].isfolder === 1 && sharesRaw[x].itemid !== 0 && wantedType === WANTET_TYPE_FOLDERS) {
            let folderId = sharesRaw[x].itemid;

            let model = createFolderFromLinkModel();
            model.name = sharesRaw[x].name;

            model.openAction = (function (y) {
                return function () {
                    openDestination(sharesRaw[y].itemid, 'folder');
                }
            })(x);

            // The only value of the array which needed to be filled with some model information during the loop.
            let inviteToFolderModel = {
                name: "Invite to Folder",
                action: (function (y) {
                    return function () {
                        openInvitationToFolder(sharesRaw[y].itemid, sharesRaw[y].name, justOutgoingShares, 666);
                    }
                })(x)
            };

            model.shareItems[0] = inviteToFolderModel;

            // Updating the function of the dw or upload according to the type of the share
            if (sharesRaw[x].isupload === 1) {
                model.stopActions.push((function (y) {
                    return function () {
                        deleteUploadLink(sharesRaw[y].linkid);
                    }
                })(x));

                let code = sharesRaw[x].code;
                let itemid = sharesRaw[x].itemid;

                model.uploadLinkAction = function () {
                    loadUploadLink(code);
                };
                model.shareItems[1].action = function () {
                    loadUploadLink(code);
                };
                model.uploadLinkStyle = SHOW_ACTIVE_LINK_ICON;


                model.downloadLinkAction = function () {
                    generateDownloadLink(itemid);
                };

                model.shareItems[2].action = function () {
                    generateDownloadLink(itemid);
                };

            } else {

                model.stopActions.push((function (y) {
                    return function () {
                        deletePublicLink(sharesRaw[y].linkid);
                    }
                })(x));

                let code = sharesRaw[x].code;
                let itemid = sharesRaw[x].itemid;

                model.downloadLinkAction = function () {
                    loadLink(code);
                };
                model.shareItems[2].action = function () {
                    loadLink(code);
                };

                model.downloadLinkStyle = SHOW_ACTIVE_LINK_ICON;

                model.shareItems[1].action = function () {
                    generateUploadLink(itemid);
                };

                model.uploadLinkAction = function () {
                    generateUploadLink(itemid);
                };
            }

            model.inviteAction = (function (y) {
                return function () {
                    openInvitationToFolder(sharesRaw[y].itemid, sharesRaw[y].name, justOutgoingShares, 666);
                }
            })(x);

            // Checking if the key match a model or its not yet defined
            if (result[sharesRaw[x].itemid] === undefined) {
                result[sharesRaw[x].itemid] = model;
            } else {

                // Model is existing. I'm updating only the necessary information
                if (sharesRaw[x].isupload === 1) {
                    result[sharesRaw[x].itemid].stopActions.push((function (y) {
                        return function () {
                            deleteUploadLink(sharesRaw[y].linkid);
                        }
                    })(x));

                    // The share is upload and I'm updating the model with upload info
                    result[sharesRaw[x].itemid].uploadLinkStyle = SHOW_ACTIVE_LINK_ICON;
                    let code = sharesRaw[x].code;

                    result[sharesRaw[x].itemid].uploadLinkAction = function () {
                        loadUploadLink(code);
                    };
                    result[sharesRaw[x].itemid].shareItems[1].action = function () {
                        loadUploadLink(code);
                    };
                } else {
                    result[sharesRaw[x].itemid].stopActions.push((function (y) {
                        return function () {
                            deletePublicLink(sharesRaw[y].linkid);
                        }
                    })(x));

                    // The share is download and I'm updating the model with download info
                    result[sharesRaw[x].itemid].downloadLinkStyle = SHOW_ACTIVE_LINK_ICON;
                    let code = sharesRaw[x].code;
                    result[sharesRaw[x].itemid].downloadLinkAction = function () {
                        loadLink(code);
                    };
                    result[sharesRaw[x].itemid].shareItems[2].action = function () {
                        loadLink(code);
                    };
                }
            }

        } else if (sharesRaw[x].folderid !== undefined && sharesRaw[x].folderid > 0 && wantedType === WANTET_TYPE_FOLDERS) {
            if (result[sharesRaw[x].folderid] !== undefined) {

                result[sharesRaw[x].folderid].inviteEmailAddresses.push(sharesRaw[x].toemail);

                if (sharesRaw[x].sharerequestid != undefined && sharesRaw[x].sharerequestid > 0) {

                    result[sharesRaw[x].folderid].stopActions.push((function (y) {
                        return function () {
                            declineShareRequestOutgoing(sharesRaw[y].sharerequestid);
                        }
                    })(x));

                } else {
                    result[sharesRaw[x].folderid].stopActions.push((function (y) {
                        return function () {
                            cancelAcceptedShare(sharesRaw[y].isba, sharesRaw[y].shareid);
                        }
                    })(x));
                }

            } else {

                let model = createFolderFromLinkModel();

                model.name = sharesRaw[x].sharename;

                model.openAction = (function (y) {
                    return function () {
                        openDestination(sharesRaw[y].folderid, 'folder');
                    }
                })(x);

                if (sharesRaw[x].sharerequestid != undefined && sharesRaw[x].sharerequestid > 0) {
                    model.stopActions.push((function (y) {
                        return function () {
                            declineShareRequestOutgoing(sharesRaw[y].sharerequestid);
                        }
                    })(x));

                } else {
                    model.stopActions.push((function (y) {
                        return function () {
                            cancelAcceptedShare(sharesRaw[y].isba, sharesRaw[y].shareid)
                        }
                    })(x));
                }

                // The only value of the array which needed to be filled with some model information from the loop.
                let inviteToFolderModel = {
                    name: "Invite to Folder",
                    action: (function (y) {
                        return function () {
                            openInvitationToFolder(sharesRaw[y].folderid, sharesRaw[y].sharename, justOutgoingShares, 666);
                        }
                    })(x)
                };

                model.uploadLinkAction = (function (y) {
                    return function () {
                        generateUploadLink(sharesRaw[y].folderid);
                    }
                })(x);

                model.downloadLinkAction = (function (y) {
                    return function () {
                        generateDownloadLink(sharesRaw[y].folderid);
                    }
                })(x);

                model.shareItems[0] = inviteToFolderModel;

                model.shareItems[1].action = (function (y) {
                    return function () {
                        generateUploadLink(sharesRaw[y].folderid);
                    }
                })(x);

                model.shareItems[2].action = (function (y) {
                    return function () {
                        generateDownloadLink(sharesRaw[y].folderid);
                    }
                })(x);

                model.inviteStyle = INVITE_STYLE_ACTIVE;

                model.inviteAction = (function (y) {
                    return function () {
                        openInvitationToFolder(sharesRaw[y].folderid, sharesRaw[y].sharename, justOutgoingShares, 666);
                    }
                })(x);

                model.inviteEmailAddresses.push(sharesRaw[x].toemail);

                result[sharesRaw[x].folderid] = model;
            }

        } else if (sharesRaw[x].isfolder !== undefined && sharesRaw[x].isfolder === 0 && sharesRaw[x].itemid !== 0 && wantedType === WANTED_TYPE_FILES) {
            // The share type here is a file. Note: Its not even a virtual folder!
            let model = createFileFromLinkModel();

            model.name = sharesRaw[x].name;
            model.icon = sharesRaw[x].icon;
            model.stopActions.push((function (y) {
                return function () {
                    deletePublicLink(sharesRaw[y].linkid);
                }
            })(x));

            model.openAction = (function (y) {
                return function () {
                    openDestination(sharesRaw[y].itemid, 'file');
                }
            })(x);

            model.shareItems[0] = createCopyLinkModel();
            let code = sharesRaw[x].code;
            model.shareItems[0].action = function () {
                loadLink(code);
            };

            model.downloadLinkAction = function () {
                loadLink(code);
            };

            result[sharesRaw[x].itemid] = model;
            // Do I have to check if there is already itemid?

        } else if (sharesRaw[x].isfolder !== undefined && sharesRaw[x].isfolder === 1 && sharesRaw[x].itemid === 0 && wantedType === WANTED_TYPE_VIRTUALFOLDERS) {
            let model = createFileFromLinkModel();
            model.name = sharesRaw[x].name;
            model.icon = sharesRaw[x].icon;
            let code = sharesRaw[x].code;

            model.stopActions.push((function (y) {
                return function () {
                    deletePublicLink(sharesRaw[y].linkid);
                }
            })(x));

            model.openAction = (function (y) {
                return function () {
                    let fullcode = LINK_INITIAL_ADDRESS_DOWNLOAD + code;
                    openVirtualFolder(fullcode);
                }
            })(x);

            model.shareItems[0] = createCopyLinkModel();

            model.shareItems[0].action = function () {
                loadLink(code);
            };

            model.downloadLinkAction = function () {
                loadLink(code);
            };

            // I'm using linkid for mapping because its the only unique property of virtual folders
            result[sharesRaw[x].linkid] = model;
            // Do I have to check if there is already itemid?
        }
    }
    return Object.values(result);
    // return result;
}

// File model for shareItems[]
function createCopyLinkModel() {
    let model = {
        name: "Copy Link",
        action: null
    };
    return model;
}

// Need to copy the DW link into clipboard and push a notification to the user
function loadLink(code) {
    link = LINK_INITIAL_ADDRESS_DOWNLOAD + code;
    mainProcess.loadTextToClipboard(link);
}

function loadUploadLink(code) {
    link = LINK_INITIAL_ADDRESS_UPLOAD + code;
    mainProcess.loadTextToClipboard(link);
}

function createFileFromLinkModel() {
    return shareModel = {
        icon: null,
        name: "",
        stopActions: [],
        shareItems: [],
        uploadLinkAction: null,
        uploadLinkStyle: HIDE_LINK_ICON,
        downloadLinkAction: null,
        downloadLinkStyle: SHOW_ACTIVE_LINK_ICON,
        inviteAction: null,
        inviteStyle: INVITE_STYLE_HIDE,
        inviteEmailAddresses: null,
        openAction: null
    };
}

function createFolderFromLinkModel() {

    let copyUpLinkModel = {};
    copyUpLinkModel.name = "Copy Upload Link";
    copyUpLinkModel.action = null;

    let copyDwLinkModel = {};
    copyDwLinkModel.name = "Copy Download Link";
    copyDwLinkModel.action = null;

    return shareModel = {
        icon: FOLDER_ICON_DEFAULT_ID,
        name: "",
        stopActions: [],
        shareItems: [
            null,
            copyUpLinkModel,
            copyDwLinkModel
        ],
        uploadLinkAction: null,
        uploadLinkStyle: SHOW_INACTIVE_LINK_ICON,
        downloadLinkAction: null,
        downloadLinkStyle: SHOW_INACTIVE_LINK_ICON,
        inviteAction: null,
        inviteStyle: INVITE_STYLE_GRAY,
        inviteEmailAddresses: [],
        openAction: null
    };
}

function generateDownloadLink(folderId) {
    mainProcess.generateFolderPublicLink(folderId, null);
}

function generateUploadLink(folderId) {
    mainProcess.generateUploadToFolderLink(folderId);
}

function createInviteFolderModel(shareid) {
    let actionModel = {};
    actionModel.isBusiness = false;
    actionModel.shareRequestId = 0;
    let model = {};
    model.shareid = shareid;
    model.icon = INCOMING_SHARES_ICON_ENVELOPE;
    model.name = "";
    model.currentPermission = INCOMING_SHARES_PERMISSION_CANVIEW;
    model.actionInfoModel = actionModel;
    model.permissionsOption = [];
    model.fontColor = SHARES_FONTCOLOR_NORMAL;
    return model;
}

function openInvitationToFolder(sharefolderId, name, sharesRaw, specialParameter = null) {
    // console.log(specialParameter);
    // Get back a proper list of share models /with permissions, names and all/
    let listShares = filterShares(sharefolderId, sharesRaw, specialParameter);
    mainProcess.openInviteFolder(sharefolderId, name, listShares);
}

function filterShares(sharefolderId, sharesRaw, specialParameter) {
    let filteredSharesList = [];
    let incomingShareRequestsList = {}
    let allIncomingPendingSharesArray = [];
    if (specialParameter === 666) {
        incomingShareRequestsList = mainProcess.getShareRequestList(OUTGOING_SHARES);
        allIncomingPendingSharesArray = Object.values(incomingShareRequestsList);
    } else if (specialParameter === 333) {
        let tempShareArray = mainProcess.getShareRequestList(OUTGOING_SHARES);
        let tempShareArray2 = mainProcess.getShareRequestList(INCOMING_SHARES);
        incomingShareRequestsList = tempShareArray.concat(tempShareArray2);
        allIncomingPendingSharesArray = Object.values(incomingShareRequestsList);
        allIncomingPendingSharesArray.forEach(function (element) {
            console.log(element);
        });
    } else {
        incomingShareRequestsList = mainProcess.getShareRequestList(INCOMING_SHARES);
        allIncomingPendingSharesArray = Object.values(incomingShareRequestsList);
    }

    sharesRaw.forEach(function (share) {

        if (share.folderid !== undefined && sharefolderId === share.folderid) {

            if (share.isba !== undefined && share.isba === 0) {

                let model = createInviteFolderModel(share.shareid);

                model.name = share.toemail;

                if (share.canmodify === 1) {
                    model.currentPermission = INCOMING_SHARES_PERMISSION_CANEDIT;
                }

                model.action = (function (share) {
                    return function () {
                        cancelAcceptedShare(share.isba, share.shareid);
                    }
                })(share)

                model.permissionsOption.push(INCOMING_SHARES_PERMISSION_CANVIEW);
                model.permissionsOption.push(INCOMING_SHARES_PERMISSION_CANEDIT);
                if (specialParameter === 333) {
                    model.fontColor = SHARES_FONTCOLOR_NORMAL;
                } else {
                    model.fontColor = SHARES_FONTCOLOR_GRAY;
                }


                filteredSharesList.push(model);
            }
        }
    });

    sharesRaw.forEach(function (share) {

        if (share.folderid !== undefined && sharefolderId === share.folderid) {
            if (share.isba !== undefined && share.isba === 1) {

                let model = createInviteFolderModel(share.shareid);
                model.name = share.toemail;
                model.icon = INCOMING_SHARES_ICON_BUSINESS;
                model.actionInfoModel.isBusiness = true;
                if (share.canmodify === 0) {
                    model.currentPermission = INCOMING_SHARES_PERMISSION_CANVIEW;
                } else if (share.canmodify === 1 && share.canmanage === 1) {
                    model.currentPermission = INCOMING_SHARES_PERMISSION_CANMANAGE;
                } else {
                    model.currentPermission = INCOMING_SHARES_PERMISSION_CANEDIT;
                }

                if (mainProcess.getUserName() === share.toemail) {
                    model.fontColor = SHARES_FONTCOLOR_GRAY;

                } else {
                    model.fontColor = SHARES_FONTCOLOR_NORMAL;

                    model.action = (function (share) {
                        return function () {
                            cancelAcceptedShare(share.isba, share.shareid);
                        }
                    })(share);

                    model.permissionsOption.push(INCOMING_SHARES_PERMISSION_CANVIEW);
                    model.permissionsOption.push(INCOMING_SHARES_PERMISSION_CANEDIT);
                    model.permissionsOption.push(INCOMING_SHARES_PERMISSION_CANMANAGE);
                }

                filteredSharesList.push(model);
            }
        }
    });

    allIncomingPendingSharesArray.forEach(function (request) {
        if (request.folderid !== undefined && sharefolderId === request.folderid) {
            let model = createInviteFolderModel(null);
            model.name = request.toemail + " (pending)";

            if (request.canmodify === 0) {
                model.currentPermission = INCOMING_SHARES_PERMISSION_CANVIEW;
            } else {
                model.currentPermission = INCOMING_SHARES_PERMISSION_CANEDIT;
            }

            model.actionInfoModel.shareRequestId = request.sharerequestid;
            // if (specialParameter === 333) {
            //     model.fontColor = SHARES_FONTCOLOR_NORMAL;
            // } else {
            //     model.fontColor = SHARES_FONTCOLOR_GRAY;
            // }
            filteredSharesList.push(model);
        }
    });

    return filteredSharesList;
}

function deletePublicLink(linkId) {
    mainProcess.deletePublicLink(linkId);
}

function deleteUploadLink(uploadLinkId) {
    mainProcess.deleteUploadLink(uploadLinkId);
}

function sortLexicographically(a, b) {
    var nameA = a.name.toUpperCase(); // ignore upper and lowercase
    var nameB = b.name.toUpperCase(); // ignore upper and lowercase
    if (nameA < nameB) {
        return -1;
    }
    if (nameA > nameB) {
        return 1;
    }
    // names must be equal
    return 0;
}

function iconUserPicker(arrayLength) {
    if (arrayLength !== 0) {
        return 'img/shares/shares-user-active.png'
    } else {
        return 'img/shares/shares-user-nonactive.png'
    }
}

function upLinkIconPicker(style) {
    if (style === 2) {
        return 'img/shares/up-link-active.png'
    } else {
        return 'img/shares/up-link-nonactive.png'
    }
}

function dwLinkIconPicker(style) {
    if (style === 2) {
        return 'img/shares/down-link-active.png'
    } else {
        return 'img/shares/down-link-nonactive.png'
    }
}

function acceptSharesIconPicker(share) {
    let iconLocation = "";
    switch (share.fromColumn.icon) {
        case 0:
            iconLocation = "img/shares/envelope.png";
            break;
        case 1:
            iconLocation = "img/shares/user.png";
            break;
        default:
            iconLocation = "img/shares/envelope.png";
            break;
    }
    return iconLocation;
}

function iconLocator(share) {
    let iconLocation = "";
    switch (share.icon) {
        case 666:
            iconLocation = 'img/shares/icons/virtual-folder.png';
            break;
        case 0:
            iconLocation = 'img/shares/icons/unknown.png';
            break;
        case 1:
            iconLocation = 'img/shares/icons/image.png';
            break;
        case 2:
            iconLocation = 'img/shares/icons/video.png';
            break;
        case 3:
            iconLocation = 'img/shares/icons/audio.png';
            break;
        case 4:
            iconLocation = 'img/shares/icons/document.png';
            break;
        case 5:
            iconLocation = 'img/shares/icons/archive.png';
            break;
        case 6:
            iconLocation = 'img/shares/icons/doc.png';
            break;
        case 7:
            iconLocation = 'img/shares/icons/presentation.png';
            break;
        case 8:
            iconLocation = 'img/shares/icons/exel.png';
            break;
        case 10:
            iconLocation = 'img/shares/icons/pages.png';
            break;
        case 11:
            iconLocation = 'img/shares/icons/keynote.png';
            break;
        case 12:
            iconLocation = 'img/shares/icons/numbers.png';
            break;
        case 13:
            iconLocation = 'img/shares/icons/odt.png';
            break;
        case 14:
            iconLocation = 'img/shares/icons/sheet.png';
            break;
        case 15:
            iconLocation = 'img/shares/icons/presentation.png';
            break;
        case 16:
            iconLocation = 'img/shares/icons/exe.png';
            break;
        case 17:
            iconLocation = 'img/shares/icons/pdf.png';
            break;
        case 18:
            iconLocation = 'img/shares/icons/html.png';
            break;
        case 19:
            iconLocation = 'img/shares/icons/trash.png';
            break;
        case 20:
            iconLocation = 'img/shares/icons/folder-linux.png';
            break;
        case 21:
            iconLocation = 'img/shares/icons/ebook.png';
            break;
        default:
            iconLocation = 'img/shares/icons/unknown.png';
            break;
    }
    return iconLocation;
}

// Incoming Shares Logic

function getMeIncomingSharesModel(type, incomingShares) {

    let result = {};

    let arrayLength = incomingShares.length;

    for (var x = 0; x < arrayLength; x++) {

        if (type === 'accepted') {

            if (result[incomingShares[x].folderid] === undefined) {
                let model = createIncomingAcceptedShareModel();
                let actionModel = createIncomingShareActionModel();

                model.name = incomingShares[x].sharename;
                model.fromColumn.name = incomingShares[x].fromemail;

                model.openAction = (function (y) {
                    return function () {
                        openDestination(incomingShares[y].folderid, 'folder');
                    }
                })(x);

                if (incomingShares[x].isba === 1) {
                    model.fromColumn.icon = INCOMING_SHARES_ICON_BUSINESS;

                    actionModel.name = "Manage";
                    // Change enable or disable the "Manage" button
                    if (incomingShares[x].canmanage === 1) {
                        model.permission = INCOMING_SHARES_PERMISSION_CANMANAGE;
                        model.actionButtonStyle = INCOMING_SHARES_BUTTON_STYLE_ENABLE;

                        actionModel.action = (function (y) {
                            return function () {
                                // I need "333" for to be able to disable the Stop button in the invite folder window
                                openInvitationToFolder(incomingShares[y].folderid, incomingShares[y].sharename, incomingShares, 333);
                            }
                        })(x);

                    } else {
                        if (incomingShares[x].canmodify === 1) {
                            model.permission = INCOMING_SHARES_PERMISSION_CANEDIT;
                        } else {
                            model.permission = INCOMING_SHARES_PERMISSION_CANVIEW;
                        }
                        model.actionButtonStyle = INCOMING_SHARES_BUTTON_STYLE_DISABLE;
                    }

                    model.action.push(actionModel);

                } else {
                    model.fromColumn.icon = INCOMING_SHARES_ICON_ENVELOPE;
                    // Change the permission from canview or canedit
                    if (incomingShares[x].canmodify === 1) {

                        model.permission = INCOMING_SHARES_PERMISSION_CANEDIT;
                    } else {
                        model.permission = INCOMING_SHARES_PERMISSION_CANVIEW;
                    }

                    model.actionButtonStyle = INCOMING_SHARES_BUTTON_STYLE_ENABLE;

                    actionModel.name = "Cancel";
                    // A function that will cancel the share. It needs to ask the user if he is sure by his choice
                    actionModel.action = (function (y) {
                        return function () {
                            cancelAcceptedShare(incomingShares[y].isba, incomingShares[y].shareid);
                        }
                    })(x);

                    model.action.push(actionModel);
                }

                result[incomingShares[x].folderid] = model;
            } else {

                if (incomingShares[x].isba === 1) {

                    if (incomingShares[x].canmanage === 1) {

                        result[incomingShares[x].folderid].fromColumn.name = incomingShares[x].fromemail;

                        result[incomingShares[x].folderid].fromColumn.icon = INCOMING_SHARES_ICON_BUSINESS;

                        result[incomingShares[x].folderid].actionButtonStyle = INCOMING_SHARES_BUTTON_STYLE_ENABLE;

                        let actionModel = createIncomingShareActionModel();
                        actionModel.name = "Manage";

                        actionModel.action = (function (y) {
                            return function () {
                                openInvitationToFolder(incomingShares[y].folderid, incomingShares[y].sharename, incomingShares, 333);
                            }
                        })(x)

                        result[incomingShares[x].folderid].action = [];
                        result[incomingShares[x].folderid].action.push(actionModel);
                    }
                }
            }

        } else {
            // THE PENDING INVITATIONS ARE MANAGED HERE

            if (incomingShares[x].isba === 0) {
                let model = createIncomingAcceptedShareModel();

                let actionAcceptModel = createIncomingShareActionModel();
                actionAcceptModel.name = "Accept";

                actionAcceptModel.action = (function (y) {
                    return function () {
                        return mainProcess.acceptShareRequest(incomingShares[y].sharerequestid);
                    }
                })(x);

                let actionDeclineModel = createIncomingShareActionModel();
                actionDeclineModel.name = "Decline";
                actionDeclineModel.action = (function (y) {
                    return function () {
                        mainProcess.declineShareRequest(incomingShares[y].sharerequestid);
                    }
                })(x);

                model.name = incomingShares[x].sharename;
                model.fromColumn.name = incomingShares[x].toemail;

                model.fromColumn.icon = INCOMING_SHARES_ICON_ENVELOPE;
                // Change the permission from canview or canedit
                if (incomingShares[x].canmodify === 1) {

                    model.permission = INCOMING_SHARES_PERMISSION_CANEDIT;
                } else {
                    model.permission = INCOMING_SHARES_PERMISSION_CANVIEW;
                }

                model.actionButtonStyle = INCOMING_SHARES_BUTTON_STYLE_ENABLE;
                model.action.push(actionAcceptModel);
                model.action.push(actionDeclineModel);

                result[incomingShares[x].folderid] = model;
            }
        }
    }
    return Object.values(result);
}

function createIncomingAcceptedShareModel() {
    let fromObject = {};
    fromObject.icon = 0;
    fromObject.name = "";

    return model = {
        name: "",
        fromColumn: fromObject,
        permission: 0,
        actionButtonStyle: 1,
        action: [],
        openAction: null
    };
}

function createIncomingShareActionModel() {
    let actionModel = {};
    actionModel.name = "";
    actionModel.action = null;
    return actionModel;
}

function cancelAcceptedShare(isBusiness, shareId) {
    mainProcess.removeShare(isBusiness, shareId);
}

function declineShareRequestOutgoing(requestId) {
    mainProcess.declineShareRequestOutgoing(requestId);
}

// Function used to display the shorter(readable) names for the shares from column.
function contactsNameFilter(sharesArray, contactsArray) {

    for (var x = 0; x < sharesArray.length; x++) {

        for (var y = 0; y < contactsArray.length; y++) {

            if (sharesArray[x].fromColumn.name === contactsArray[y].mail) {
                if (contactsArray[y].name.trim().length > 0) {
                    sharesArray[x].fromColumn.name = contactsArray[y].name;
                }
                break;
            }
        }
    }
}

function openDestination(id, type) {
    mainProcess.openDestination(id, type);
}

function openVirtualFolder(url) {
    mainProcess.openExternalLocation(url);
}