var { ipcRenderer, remote } = require('electron');
const mainProcess = remote.require('./main');

var extAccountScope = null;

document.addEventListener('dragover', function (event) {
    event.preventDefault();
    return false;
}, false);

document.addEventListener('drop', function (event) {
    event.preventDefault();
    return false;
}, false);

var InviteFolderApp = angular.module('InviteFolderApp', []);

InviteFolderApp.controller('IFController', function ($scope) {

    var vm = this;
    extAccountScope = $scope;
    vm.showSuggestions = false;
    vm.BusinessOptions = [{ name: "can view", id: 1 }, { name: "can edit", id: 2 }, { name: "can manage", id: 3 }];
    vm.NonBusinessoptions = [{ name: "can view", id: 1 }, { name: "can edit", id: 2 }];
    vm.isUserBusiness = mainProcess.isUserBusiness();

    if (vm.isUserBusiness) {
        vm.invitePermissionLevel = vm.BusinessOptions[0];
    } else {
        vm.invitePermissionLevel = vm.NonBusinessoptions[0];
    }

    vm.folderName = null;
    vm.folderId = 0;
    vm.listInvitations = null;
    vm.contactsArray = mainProcess.getContactsList();
    vm.suggestionArray = [];
    vm.disableSendInvitation = true;
    vm.iconPicker = iconPicker;
    vm.permissionConverter = permissionConverter;

    vm.changePermission = function (invite) {
        mainProcess.modifyShare(invite);
        initiateLoading(1000);
    }

    vm.closeInviteFolderWindow = function () {
        ipcRenderer.send('close-invite-folder-window');
    }

    vm.submitInvitation = function (contact, permissionLevel, message) {

        if (message === undefined) {
            message = "";
        }
        let permission = permissionParamConv(permissionLevel);
        let result = 0;
        if (contact === undefined) {
            result = mainProcess.shareFolder(vm.folderId, vm.folderName, vm.email, message, permission);
        } else {
            if ((contact.teamid > 0) || (contact.name === "Everyone" && contact.teamid === 0)) {
                result = mainProcess.accountTeamShare(vm.folderId, vm.folderName, contact.teamid, message, permission)
            } else {
                result = mainProcess.shareFolder(vm.folderId, vm.folderName, contact.mail, message, permission);
            }

        }
        if (result === 0) {
            initiateLoading(1000);
            vm.email = "";
            vm.message = "";
            vm.disableSendInvitation = true;
        }
    }

    vm.stopAccess = function (invite) {
        let confirmation = mainProcess.stopShareConfirmation("invite-folder", invite.name);
        if (confirmation) {
            if (invite.actionInfoModel.shareRequestId !== 0) {
                mainProcess.declineShareRequestOutgoing(invite.actionInfoModel.shareRequestId);
            } else if (invite.actionInfoModel.isBusiness) {
                mainProcess.removeShare(1, invite.shareid);
            } else {
                mainProcess.removeShare(0, invite.shareid);
            }
        }
    }

    vm.contactNameToDisplay = function (contact) {
        let name = contact.name;
        if (name.trim().length > 0) {
            return name;
        } else {
            return contact.mail;
        }
    }

    vm.userACChoice = function (contact) {
        let name = contact.name;
        if (name.trim().length > 0) {
            vm.email = name;
        } else {
            vm.email = contact.mail;
        }
        vm.showSuggestions = false;
        vm.contactChoice = contact;
    }

    vm.activityInitiated = function (e, text) {
        if (vm.disableSendInvitation) {
            vm.disableSendInvitation = false;
        }
        let charCode = (e.which) ? e.which : e.keyCode;

        if (text == undefined) {
            text = "";
        } else {
            text = text.toLowerCase();
        }

        if (vm.email == undefined) {
            vm.email = "";
        } else {
            vm.email = vm.email.toLowerCase();
        }
        if (charCode === 8 || charCode === 46) {
            filterContactList(vm.email);

        } else {
            // let inputResult = text + String.fromCharCode(charCode).toLowerCase();
            filterContactList(text);
        }
    }

    function filterContactList(inputResult) {
        if (inputResult.length > 2) {
            vm.suggestionArray = [];
            vm.contactsArray.forEach(function (contact) {
                if ((contact.name).toLowerCase().indexOf(inputResult) > -1 || (contact.mail).toLowerCase().indexOf(inputResult) > -1) {
                    vm.suggestionArray.push(contact);
                }
            });
        } else {
            vm.suggestionArray = [];
            vm.showSuggestions = false;
        }

        if (vm.suggestionArray.length > 0) {
            vm.showSuggestions = true;
        } else {
            vm.showSuggestions = false;
        }
    }
});

// args[0] == folderId, args[1] == folderName, args[2] == listInvited
ipcRenderer.on('folder-info', (event, args) => {

    if (extAccountScope != null) {
        extAccountScope.$apply(function () {
            extAccountScope.vm.folderId = args[0];
            extAccountScope.vm.folderName = args[1];
            extAccountScope.vm.listInvitations = args[2];
            contactsNameFilterInviteFolder(extAccountScope.vm.listInvitations, extAccountScope.vm.contactsArray);
        })
    }
});

function contactsNameFilterInviteFolder(sharesArray, contactsArray) {

    for (var x = 0; x < sharesArray.length; x++) {

        for (var y = 0; y < contactsArray.length; y++) {

            if (sharesArray[x].actionInfoModel.isBusiness) {

                if (sharesArray[x].name === contactsArray[y].mail) {
                    if (contactsArray[y].name.trim().length > 0) {
                        sharesArray[x].name = contactsArray[y].name;
                    }
                    break;
                }
            }

        }
    }
}

function permissionParamConv(permission) {
    if (permission === 1) {
        return 1;
    } else if (permission === 2) {
        return 15;
    } else {
        return 31;
    }
}

function permissionConverter(permission) {
    if (permission === 0) {
        return "can view";
    } else if (permission === 1) {
        return "can edit";
    } else {
        return "can manage";
    }
}

function iconPicker(invite) {
    let iconLocation = "";
    switch (invite.icon) {
        case 0:
            iconLocation = "img/shares/envelope.png";
            break;
        case 1:
            iconLocation = "img/shares/user.png";
            break;
        default:
            iconLocation = "img/shares/envelope.png";
            break;
    }

    return iconLocation;
}

function initiateLoading(spinningTime) {
    document.body.classList.add('loading');
    setTimeout(
        function () {
            document.body.classList.remove('loading');
        }, spinningTime);
}