//include the modules we need from electron and node.js
var { app, BrowserWindow, ipcMain, Tray, Menu, shell, dialog, clipboard } = require('electron');
var log = require('electron-log');
var eNotify = null;

//var request = require('request');
const updater = require('electron-simple-updater');
var path = require('path');
const exec = require('child_process').exec;
// const fork = require('child_process').fork;
const { spawn } = require('child_process');
var ref = require('ref');
var ffi = require('ffi');
var StructType = require("ref-struct");
var ArrayType = require('ref-array');
var UnionType = require('ref-union');
var messenger = require('messenger');
var fs = require('fs-extra');
var AutoLaunch = require('auto-launch');
const os = require('os');
const isOnline = require('is-online');

const INCOMING_SHARES = 1;
const OUTGOING_SHARES = 0;
const LINK_INITIAL_ADDRESS_DOWNLOAD = "https://my.pcloud.com/publink/show?code=";
const LINK_INITIAL_ADDRESS_UPLOAD = "https://my.pcloud.com/#page=puplink&code=";

//create a C-like struct
var contact_info = StructType({
    teamid: 'uint64',
    mail: 'string',
    name: 'string',
    type: 'uint32'
});

var pcontacts_list = StructType({
    entrycnt: 'uint32',
    entries: ArrayType(contact_info)
});

var pstatus = StructType({
    downloadstr: 'string',
    uploadstr: 'string',
    bytestoupload: 'uint64',
    bytestouploadcurrent: 'uint64',
    bytesuploaded: 'uint64',
    bytestodownload: 'uint64',
    bytestodownloadcurrent: 'uint64',
    bytesdownloaded: 'uint64',
    status: 'uint32',
    filestoupload: 'uint32',
    filesuploading: 'uint32',
    uploadspeed: 'uint32',
    filestodownload: 'uint32',
    filesdownloading: 'uint32',
    downloadspeed: 'uint32',
    remoteisfull: 'uint8',
    localisfull: 'uint8'
});

var psync_folder = StructType({
    localname: 'char *',
    localpath: 'char *',
    remotename: 'char *',
    remotepath: 'char *',
    folderid: 'uint64',
    syncid: 'uint32',
    synctype: 'uint32'
});

var psync_share = StructType({
    shareid: 'uint64',
    folderid: 'uint64',
    created: 'size_t',
    userid: 'uint64',
    fromemail: 'string',
    toemail: 'string',
    sharename: 'string',
    permissions: 'uchar',
    canread: 'uchar',
    cancreate: 'uchar',
    canmodify: 'uchar',
    candelete: 'uchar',
    canmanage: 'uchar',
    isba: 'uchar',
    isteam: 'uchar'
})

var psync_sharerequest = StructType({
    sharerequestid: 'uint64',
    folderid: 'uint64',
    created: 'size_t',
    userid: 'uint64',
    toemail: 'string',
    sharename: 'string',
    message: 'string',
    permissions: 'uchar',
    canread: 'uchar',
    cancreate: 'uchar',
    canmodify: 'uchar',
    candelete: 'uchar',
    isba: 'uchar',
});

var link_info_t = StructType({
    linkid: 'uint64',
    name: 'string',
    code: 'string',
    comment: 'string',
    traffic: 'uint64',
    maxspace: 'uint64',
    downloads: 'uint64',
    created: 'uint64',
    modified: 'uint64',
    itemid: 'uint64',
    isfolder: 'uint8',
    isupload: 'uint8',
    icon: 'uint8'
});

var psync_folder_list_t = StructType({
    foldercnt: 'size_t',
    folders: ArrayType(psync_folder)
});

var psync_share_list_t = StructType({
    sharecnt: 'size_t',
    shares: ArrayType(psync_share)
});

var psync_sharerequest_list_t = StructType({
    sharerequestcnt: 'size_t',
    sharerequests: ArrayType(psync_sharerequest)
});

var plink_info_list_t = StructType({
    entrycnt: 'uint32',
    entries: ArrayType(link_info_t)
});

var pfile = StructType({
    fileid: 'uint64',
    size: 'uint64'
});

var pfolder = StructType({
    folderid: 'uint64',
    cansyncup: 'uint8',
    cansyncdown: 'uint8',
    canshare: 'uint8',
    isencrypted: 'uint8'
});

var psync_file_event = StructType({
    fileid: 'uint64',
    name: 'string',
    localpath: 'string',
    remotepath: 'string',
    syncid: 'uint32'
});

var psync_folder_event = StructType({
    folderid: 'uint64',
    name: 'string',
    localpath: 'string',
    remotepath: 'string',
    syncid: 'uint32'
});

var psync_share_event = StructType({
    folderid: 'uint64',
    sharename: 'string',
    toemail: 'string',
    fromemail: 'string',
    message: 'string',
    userid: 'uint64',
    shareid: 'uint64',
    sharerequestid: 'uint64',
    created: 'size_t',
    canread: 'uchar',
    cancreate: 'uchar',
    canmodify: 'uchar',
    candelete: 'uchar',
    canmanage: 'uchar'
});

var psync_eventdata = UnionType({
    file: psync_file_event,
    folder: psync_folder_event,
    share: psync_share_event,
    ptr: 'void'
});

var pentry = StructType({
    name: 'string',
    union: UnionType({
        folder: pfolder,
        file: pfile
    }),
    namelen: 'uint16',
    isfolder: 'uint8'
});

var pfolder_list_t = StructType({
    entrycnt: 'size_t',
    entries: ArrayType(pentry)
});

// INITIATE VARIABLES

const oneHourInMilisecs = 3600000;
const sixHoursInMilisecs = 21600000;
const dayInMilisecs = 86400000;
const oneWeekInMilisecs = 604800000;
const neverRemindForUpdate = 1;

var pcloudAutoLauncher = null;
var MyLibrary = null;
var isAppReady = false;
var unlinked = false;
var shares = null;
var cryptoStatus = null;
var currentOpenTab = "crypto";
var releaseNotesOfNewVersion = "";
var updateVersionNumber = "";
var manualUpdateCheckRequested = false;
var isDownloadingInProgress = false;
var newUpdateDownloaded = false;

/* CRYPTO STATUS 
    showTrial = 0;
    trialExpired = 1;
    showSetup = 2;
    cryptoActive = 3;
    cryptoActiveTrialMode = 4;
*/

// Initialize the Fb library

var fbCallback = ffi.Callback('void', ['int', 'string', 'string'], function (result, msg, token) {

    switch (result) {
        case 0:
            break;
        case 42:
            // 42 means a new account will be created. Needs to open a window with the "msg" as this is the mail to be registered.
            // msg = email
            let currentUser = getUserName();
            if (currentUser == null || currentUser == "") {
                MyLibrary.psync_set_string_value('username', msg);
                MyLibrary.psync_set_string_value('fb_access_token', token);
                createRegisterConfirmationWindow();
                break;
            }
            else if (currentUser != msg) {
                MyLibrary.psync_set_string_value('fb_access_token', "");
                showError('This Facebook account is not linked to your current pCloud account. You can link both accounts in Settings on my.pcloud.com or Unlink to continue with this Facebook account.');
                break;
            } else {
                MyLibrary.psync_set_string_value('username', msg);
                MyLibrary.psync_set_string_value('fb_access_token', token);
                createRegisterConfirmationWindow();
                break;
            }
        case 43:
            MyLibrary.psync_set_string_value('fb_access_token', "");
            showError(msg);
            break;
        case 1033:
            // No permission to facebook to share the email with our app. Need to open provideEmailWindow
            // msg = 'Please provide 'mail'.'
            MyLibrary.psync_set_string_value('fb_access_token', token);
            createProvideEmailWindow();
            break;

        case 2262:
            sendTokenInfo();
            var result = MyLibrary.fb_authorize(fbCallback);
            shell.openExternal(result);
            break;
        default:
            showError(msg);
            break;
    }

});

// Initialize the C library



var pevent_callback = ffi.Callback('void', ['uint32', ref.refType(psync_eventdata)], function (eventType, eventData) {
    try {
        if (!ref.isNull(eventData)) {
            // console.log("not null event")
            if (inviteFolderIsOpened()) {
                let derefShare = eventData.deref().share;

                let justOutgoingShares = getShareList(OUTGOING_SHARES);
                let justIncomingShares = getShareList(INCOMING_SHARES);
                let allShares = justOutgoingShares.concat(justIncomingShares);

                // inviteFolderWindow.reload();
                // inviteFolderWindow.close();
                appWindow.webContents.send('open-invitation-to-folder', [derefShare.folderid, derefShare.sharename, allShares, 333]);
                appWindow.webContents.send('reload-shareRoute-page');

            } else if (currentOpenTab === "share") {
                setTimeout(function () {
                    if (appWindow != null && !appWindow.isDestroyed()) {
                        appWindow.webContents.send('reload-share-page');
                    }
                }, 1000);
                // appWindow.reload();
                // appWindow.webContents.executeJavaScript('setActive("shares");');
            }
        }

    } catch (error) {
        console.log('ERROR -----> ' + error);
    }
});

/*
READY                   0
DOWNLOADING             1
UPLOADING               2
DOWNLOADINGANDUPLOADING 3
LOGIN_REQUIRED          4
BAD_LOGIN_DATA          5
BAD_LOGIN_TOKEN         6
ACCOUNT_FULL            7
DISK_FULL               8
PAUSED                  9
STOPPED                10
OFFLINE                11
CONNECTING             12
SCANNING               13
USER_MISMATCH          14
ACCOUT_EXPIRED         15
*/

var callback = ffi.Callback('void', ['int', 'string', 'string', 'string'], function (status, stat_string, down_string, up_string) {

    // let d = new Date();
    // let n = d.toLocaleTimeString();
    // console.log('STATUS -> ' + status);
    // console.log("STATUS IS -> " + status + " seconds - " + n);
    switch (status) {

        case 0:
            if (!isAppReady && !unlinked) {
                appLoader();
            }
            else {
                if (appWindow != null && !appWindow.isDestroyed()) {
                    appWindow.webContents.send('dw-status', down_string.toString());
                    appWindow.webContents.send('up-status', up_string.toString());
                }
            }
            break;
        case 1:
            if (!isAppReady && !unlinked) {
                appLoader();
            }
            else {
                if (appWindow != null && !appWindow.isDestroyed()) {
                    appWindow.webContents.send('dw-status', down_string.toString());
                }
            }
            break;

        case 2:
            if (!isAppReady && !unlinked) {
                appLoader();
            }
            else {
                if (appWindow != null && !appWindow.isDestroyed()) {
                    appWindow.webContents.send('up-status', up_string.toString());
                }
            }
            break;

        case 3:
            if (!isAppReady && !unlinked) {
                appLoader();
            }
            else {
                if (appWindow != null && !appWindow.isDestroyed()) {
                    appWindow.webContents.send('up-status', up_string.toString());
                    appWindow.webContents.send('dw-status', down_string.toString());
                }
            }
            break;

        case 4:
            openLoginWindow();
            break;

        case 5:
            MyLibrary.logout();
            openDialogError();
            break;

        case 6:
            if (appWindow != null && !appWindow.isDestroyed() && unlinked) {
                appWindow.close();
                backgroundWindow.close();
                trayIcon.destroy()
                if (isAppReady) {
                    isAppReady = false;
                }

            } else if (appWindow != null && !appWindow.isDestroyed()) {
                appWindow.close();
                backgroundWindow.close();
                trayIcon.destroy()
                if (isAppReady) {
                    isAppReady = false;
                }
                showError('Your session has expired. Please login again.')
            }
            openLoginWindow();
            break;
        case 11:
            eNotify.notify({ title: 'No internet connection!', text: 'Please check your internet connection!' });
            // showError('No internet connection!');
            break;
        case 13:
            if (!isAppReady) {
                appLoader();
            }
            else {
                if (appWindow != null && !appWindow.isDestroyed()) {
                    appWindow.webContents.send('dw-status', down_string.toString());
                    appWindow.webContents.send('up-status', up_string.toString());
                }
            }
            break;
        default:
    }
});

function initLibrary() {
    MyLibrary = ffi.Library(path.join(__dirname, 'app/libs/libpcloudcc_lib'), {

        'psync_set_software_string': ['void', ['char *']],

        'get_token': ['string', []],

        'psync_fs_pause_until_login': ['void', []],
        'psync_fs_isstarted': ['int', []],
        'psync_fs_start': ['int', []],
        'psync_fs_stop': ['void', []],
        'psync_fs_get_path_by_folderid': ['string', ['uint64']],
        'psync_get_path_by_fileid': ['string', ['uint64', 'size_t']],
        'psync_fs_getmountpoint': ['string', []],

        'unlinklib': ['void', []],
        'logout': ['void', []],
        'login': ['int', ['char *', 'char *', 'int']],

        'psync_get_uint_value': ['uint64', ['string']],
        'psync_set_uint_value': ['void', ['string', 'uint64']],
        'psync_get_bool_value': ['int', ['string']],
        'psync_get_bool_setting': ['int', ['string']],
        'psync_set_bool_setting': ['int', ['string', 'int']],

        'psync_register': ['int', ['char *', 'char *', 'int', 'char **']],
        'psync_verify_email': ['int', ['char **']],
        'psync_lost_password': ['int', ['char *', 'char **']],
        'psync_change_password': ['int', ['char *', 'char *', 'char **']],

        'psync_get_username': ['string', []],
        'psync_set_string_value': ['void', ['string', 'string']],
        'psync_get_string_value': ['string', ['char *']],
        'psync_get_int_setting': ['int64', ['string']],
        'psync_get_uint_setting': ['uint64', ['string']],
        'psync_set_int_setting': ['int64', ['string', 'int64']],
        'psync_set_uint_setting': ['uint64', ['string', 'uint64']],
        'psync_get_string_setting': ['string', ['string']],

        'init': ['int', []],
        'set_status_callback': ['void', ['pointer']],
        'set_event_callback': ['void', ['pointer']],

        'psync_list_contacts': [ref.refType(pcontacts_list), []],
        'psync_list_myteams': [ref.refType(pcontacts_list), []],

        'psync_crypto_setup': ['int', ['string', 'string']],
        'psync_crypto_get_hint': ['int', ['char **']],
        'statrt_crypto': ['int', ['char *']],
        'stop_crypto': ['int', []],
        'psync_crypto_stop': ['int', []],
        'psync_crypto_isstarted': ['int', []],
        'psync_crypto_mkdir': ['int', ['uint64', 'string', 'char **', 'uint64 *']],
        'psync_crypto_issetup': ['int', []],
        'psync_crypto_hassubscription': ['int', []],
        'psync_crypto_isexpired': ['int', []],
        'psync_crypto_expires': ['size_t', []],
        'psync_crypto_reset': ['int', []],
        'psync_password_quality10000': ['int', ['string']],

        'finalize': ['int', []],
        'psync_destroy': ['void', []],

        'psync_add_sync_by_path': ['uint32', ['char *', 'char *', 'uint32']],
        'psync_delete_sync': ['int', ['uint32']],
        'psync_get_sync_list': [ref.refType(psync_folder_list_t), []],
        'psync_list_shares': [ref.refType(psync_share_list_t), ['int']],
        'psync_list_sharerequests': [ref.refType(psync_sharerequest_list_t), ['int']],
        'psync_accept_share_request': ['int', ['uint64', 'uint64', 'string', 'char **']],
        'psync_decline_share_request': ['int', ['uint64', 'char **']],
        'psync_cancel_share_request': ['int', ['uint64', 'char **']],
        'psync_remove_share': ['int', ['uint64', 'char **']],
        'psync_account_stopshare': ['int', ['uint64', 'char **']],
        'psync_account_teamshare': ['int', ['uint64', 'string', 'uint64', 'string', 'uint32', 'char **']],
        'psync_modify_share': ['int', ['uint64', 'uint32', 'char **']],
        'psync_share_folder': ['int', ['uint64', 'string', 'string', 'string', 'uint32', 'char **']],

        'psync_file_public_link': ['int64', ['string', 'char **', 'char **']],
        'psync_folder_public_link': ['int64', ['string', 'char **', 'char **']],
        'psync_delete_link': ['int', ['int64', 'char **']],
        'psync_delete_upload_link': ['int', ['int64', 'char **']],
        'psync_upload_link': ['int64', ['string', 'string', 'char **', 'char **']],
        'psync_tree_public_link': ['int64', ['string', 'string', ArrayType('string'), 'int', ArrayType('string'), 'int', 'char **', 'char **']],

        'psync_list_remote_folder_by_folderid': [ref.refType(pfolder_list_t), ['uint64', 'uint32']],

        'psync_list_links': [ref.refType(plink_info_list_t), ['char **']],

        'psync_stat_path': [ref.refType(pentry), ['string']],

        'fb_authorize': ['string', ['pointer']],
        'fb_login': ['int', ['char*', 'char **']],
        'fb_login_register': ['int', ['string', 'string', 'int', 'char **']],

        'psync_fs_clean_read_cache': ['void', []],
        'psync_fs_move_cache': ['int', ['string']],

        'psync_set_bool_value': ['void', ['string', 'int']]
    });
}

initLibrary();

//get the list of synced folders
function getSyncList() {
    var synclist = MyLibrary.psync_get_sync_list();
    //get the number of syncs first
    var count = synclist.deref().foldercnt;
    psync_folder_list_t = StructType({
        foldercnt: 'size_t',
        folders: ArrayType(psync_folder, count)
    });
    //then, get the whole list as an array of objects
    initLibrary();
    synclist = MyLibrary.psync_get_sync_list();
    var syncFoldFromBuf = synclist.deref().folders;
    var syncedFoldersList = [];
    for (var i = 0; i < count; i++) {
        var sync = {
            localname: ref.readCString(syncFoldFromBuf[i].localname),
            localpath: ref.readCString(syncFoldFromBuf[i].localpath),
            remotename: ref.readCString(syncFoldFromBuf[i].remotename),
            remotepath: ref.readCString(syncFoldFromBuf[i].remotepath),
            folderid: syncFoldFromBuf[i].folderid,
            syncid: syncFoldFromBuf[i].syncid,
            synctype: syncFoldFromBuf[i].synctype
        };
        syncedFoldersList.push(sync);
    }
    return syncedFoldersList;
}

//EXPORTS TO A RENDERER PROCESS
exports.getSyncList = function () {
    return getSyncList();
};

exports.getUsedQuota = function () {
    return getUsedQuota();
};

exports.getTotalQuota = function () {
    return getTotalQuota();
}

exports.getIfUserIsPremium = function () {
    return getIfUserIsPremium();
}

exports.getUserName = function () {
    return getUserName();
}

exports.getAuthToken = function () {
    return getAuthToken();
}

exports.isUserBusiness = function () {
    return isUserBusiness();
}

exports.getCompanyName = function () {
    return getCompanyName();
}

exports.cryptoStarted = function () {
    return cryptoStarted();
}

exports.cryptoPassDifficulty10000 = function (password) {
    return cryptoPassDifficulty10000(password);
}

exports.cryptoPassMemoryConfirmation = function () {
    return cryptoPassMemoryConfirmation();
}

exports.resetCryptoPassConfirmation = function () {
    return resetCryptoPassConfirmation();
}

exports.resetCryptoPass = function () {
    return resetCryptoPass();
}

exports.cryptoSetup = function (cryptoPass, hint) {
    return cryptoSetup(cryptoPass, hint);
}

exports.showError = function (message) {
    return showError(message);
}

exports.showMessage = function (message) {
    return showMessage(message);
}

exports.getIfUserNeedVerification = function () {
    return getIfUserNeedVerification();
}

exports.updateInfo = function (infoType, value) {
    return updateInfo(infoType, value);
}

exports.getAllSettings = function () {
    return getAllSettings();
}

exports.getCurrentCacheLoc = function () {
    return getCurrentCacheLoc();
}

exports.clearCache = function () {
    return clearCache();
}

exports.changeCacheLocation = function () {
    return changeCacheLocation();
}

// exports.isAutoStartEnabled = function () {
//     return isAutoStartEnabled();
// }

exports.setBoolValue = function (stringKey, intValue) {
    return setBoolValue(stringKey, intValue);
}

exports.getBoolValue = function (stringKey) {
    return getBoolValue(stringKey);
}

exports.checkCryptoStatus = function () {
    return checkCryptoStatus();
}

exports.unlockCrypto = function (cryptoPass) {
    return unlockCrypto(cryptoPass);
}

exports.lockCryptoFolder = function () {
    return lockCryptoFolder();
}

exports.getCryptoExpirationDaysString = function () {
    return getCryptoExpirationDaysString();
}

exports.cryptoExpiredGracePeriod = function () {
    return cryptoExpiredGracePeriod();
}

// exports.disableAutoStart = function () {
//     return disableAutoStart();
// }

// exports.enableAutoStart = function () {
//     return enableAutoStart();
// }

exports.getShareList = function (type) {
    return getShareList(type);
}

exports.modifyShare = function (share) {
    return modifyShare(share);
}

exports.shareFolder = function (folderId, folderName, email, message, permission) {
    return shareFolder(folderId, folderName, email, message, permission);
}

exports.accountTeamShare = function (folderId, folderName, teamId, message, permission) {
    return accountTeamShare(folderId, folderName, teamId, message, permission);
}

exports.getShareRequestList = function (type) {
    return getShareRequestList(type);
}

exports.loadTextToClipboard = function (text) {
    return loadTextToClipboard(text);
}

exports.generateUploadLinkFromPicker = function () {
    return generateUploadLinkFromPicker();
}

exports.generateDownloadFileLinkFromPicker = function () {
    return generateDownloadFileLinkFromPicker();
}

exports.generateDownloadFolderLinkFromPicker = function () {
    return generateDownloadFolderLinkFromPicker();
}

exports.generateVirtualFolder = function (vFName, elementsType, arrayPaths) {
    return generateVirtualFolder(vFName, elementsType, arrayPaths);
}

exports.getContactsList = function () {
    return getContactsList();
}

exports.stopShareConfirmation = function (shareTabName, account) {
    return stopShareConfirmation(shareTabName, account);
}

exports.getMyTeamsList = function () {
    return getMyTeamsList();
}

exports.getPublicLinks = function () {
    return getPublicLinks();
}

exports.openInviteFolder = function (folderId, folderName, shareList) {
    return openInviteFolder(folderId, folderName, shareList);
}

exports.openChooseFolderToShareDialog = function () {
    return openChooseFolderToShareDialog();
}

exports.deletePublicLink = function (linkId) {
    return deletePublicLink(linkId);
}

exports.deleteUploadLink = function (uploadLinkId) {
    return deleteUploadLink(uploadLinkId);
}

exports.generateUploadToFolderLink = function (folderId) {
    return generateUploadToFolderLink(folderId);
}

exports.generateFolderPublicLink = function (folderId, path) {
    return generateFolderPublicLink(folderId, path);
}

exports.acceptShareRequest = function (requestid) {
    return acceptShareRequest(requestid);
}

exports.declineShareRequest = function (requestid) {
    return declineShareRequest(requestid);
}

exports.declineShareRequestOutgoing = function (requestid) {
    return declineShareRequestOutgoing(requestid);
}

exports.removeShare = function (isBusiness, shareId) {
    return removeShare(isBusiness, shareId);
}

exports.openDestination = function (id, type) {
    return openDestination(id, type);
}

exports.openExternalLocation = function (url) {
    return openExternalLocation(url);
}

exports.setCurrentOpenTab = function (tab) {
    return setCurrentOpenTab(tab);
}

exports.getReleaseNotes = function () {
    return getReleaseNotes();
}

exports.getUpdateVersion = function () {
    return getUpdateVersion();
}

exports.newVersionChecker = function () {
    return newVersionChecker();
}

exports.initiateUpdate = function () {
    return initiateUpdate();
}

exports.setNextNotificationTime = function (argument) {
    return setNextNotificationTime(argument);
}

exports.getUpdaterInterval = function () {
    return getUpdaterInterval();
}

exports.getNextNotificationTime = function () {
    return getNextNotificationTime();
}

exports.initiateUpdateNotification = function () {
    return initiateUpdateNotification();
}

exports.checkForUpdates = function () {
    return checkForUpdates();
}

exports.isDownloadingInProgressFunc = function () {
    return isDownloadingInProgressFunc();
}

exports.manualCheckForUpdate = function () {
    return manualCheckForUpdate();
}

exports.manualCheckForUpdate = function () {
    return manualCheckForUpdate();
}

exports.testFunction = function () {
    return testFunction();
}

exports.updateAlreadyDownloaded = function () {
    return updateAlreadyDownloaded();
}

exports.restartApplication = function () {
    return restartApplication();
}

var testFunction = function () {

    // var choice = require('electron').dialog.showMessageBox(appWindow,
    //     {
    //         type: 'question',
    //         buttons: ['Restart', 'Later'],
    //         title: 'Update installed',
    //         message: 'Restart application to apply the new update?\n\nNote: If the application does not start automatically, please start it manually.'
    //     });
}

// UPDATE THE RENDERER PROCESS FROM BACKGROUND PROCESS
function updateInfo(infoType, value) {
    if (appWindow !== null && !appWindow.isDestroyed()) {
        appWindow.webContents.send('update-info', [infoType, value]);
    }
}

var appLoader = function () {
    isAppReady = true;
    cryptoStatus = checkCryptoStatus();
    // configUpdaterSettings();
    // createBackgroundWorker();
    if (loginWindow !== null && !loginWindow.isDestroyed()) {
        loginWindow.destroy();
    }
    if (registerWindow != null && !registerWindow.isDestroyed()) {
        registerWindow.destroy();
    }

    initTray();

    if (getStringSetting("auto-start-string") === null || getStringSetting("auto-start-string") === "1") {
        InitialLoadOpenCryptoPage();
    }

    if (appWindow != null && !appWindow.isDestroyed()) {
        appWindow.webContents.send('dw-status', 'Everything Downloaded');
        appWindow.webContents.send('up-status', 'Everything Uploaded');
    }

    if (getStringSetting("auto-start-string") === null || getStringSetting("auto-start-string") === "1") {
        try {
            shell.showItemInFolder(process.env['HOME'] + '/pCloudDrive/./');
        } catch (error) {
            log.info('ERROR -> ' + error);
        }
    }

    initAutostartSettings();
    checkAndSuggestUnlockCrypto();
    initiateFirstUpdateNotifyChecker();
}

var createBackgroundWorker = function () {
    if (!backgroundWindow || backgroundWindow.isDestroyed()) {
        backgroundWindow = new BrowserWindow({ show: false });
        backgroundWindow.loadURL('file://' + __dirname + '/app/background.html');
        // backgroundWindow.webContents.openDevTools({ detach: true });
    }
}

var getIfUserNeedVerification = function () {
    let result = MyLibrary.psync_get_bool_value("emailverified");

    if (result == 0) {
        return true;
    } else {
        return false;
    }
}

// UPDATER

var initiateFirstUpdateNotifyChecker = function () {
    let currentUser = getUserName();
    let updateInterval = MyLibrary.psync_get_uint_value('updater-interval');
    if (currentUser !== null && currentUser !== "") {
        if (newVersionChecker() && updateInterval !== neverRemindForUpdate) {
            initiateUpdateNotification();
        }
    }
}

function choiceDialogForAvailableUpdate() {
    var choice = require('electron').dialog.showMessageBox(appWindow,
        {
            type: 'question',
            buttons: ['Update', 'Cancel'],
            title: 'New update available',
            message: 'Download and install the new update?'
        });
    if (choice === 0) {
        updater.downloadUpdate();

    } else {
        return false;
    }
}

var restartApplication = function () {
    backgroundWindow.webContents.send('initiate-restart');
    appWindow.webContents.send('initiateLoading');
    setTimeout(function () {
        app.quit();
    }, 1500);
}

function updateDownloadedAskForRestart() {
    var choice = require('electron').dialog.showMessageBox(appWindow,
        {
            type: 'question',
            buttons: ['Restart', 'Later'],
            title: 'Update installed',
            message: 'Restart application to apply the new update?\n\n Note: If application does not start automatically, please start it manually.'
        });
    if (choice === 0) {
        restartApplication();
    } else {
        showMessage("Update will apply after application restart");
        if (appWindow !== null && !appWindow.isDestroyed() && currentOpenTab === 'about') {
            appWindow.webContents.send('show-only-restart-option');
        }
        newUpdateDownloaded = true;
    }
}

var initiateUpdateNotification = function () {
    let currentUser = getUserName();
    if (currentUser !== null || currentUser !== "") {
        let versionString = 'New Version: ' + updateVersionNumber;
        eNotify.notify({
            title: 'New version available', text: 'Click here for more info!', onClickFunc: function () {
                if (!appWindow || appWindow.isDestroyed()) {
                    openApp('#/about');
                }
                else {
                    appWindow.webContents.executeJavaScript(
                        'document.getElementById("about").click(); ' +
                        'setActive("about");');
                    appWindow.show();
                }
            }
        });
    }
}

var configUpdaterSettings = function () {
    let now = Date.now();
    let updateInterval = MyLibrary.psync_get_uint_value('updater-interval');
    let nextNotificationTime = MyLibrary.psync_get_uint_value('next-notify-time');

    // console.log("Date.now() = " + Date.now());
    // console.log("updateInterval = " + updateInterval);
    // console.log("nextNotificationTime = " + nextNotificationTime);

    if (updateInterval !== null && updateInterval === 0) {
        MyLibrary.psync_set_uint_value('updater-interval', oneHourInMilisecs);
        // MyLibrary.psync_set_uint_value('updater-interval', 40000);
    }

    if (nextNotificationTime !== 0 && nextNotificationTime <= now) {
        if (newVersionChecker()) {
            initiateUpdateNotification();
        }
        nextNotificationTime = now + updateInterval;
        MyLibrary.psync_set_uint_value('next-notify-time', nextNotificationTime);
    }

    if ((nextNotificationTime !== null && nextNotificationTime === 0)) {
        nextNotificationTime = now + updateInterval;
        MyLibrary.psync_set_uint_value('next-notify-time', nextNotificationTime);
    }
}

var setNextNotificationTime = function (argument) {
    let now = Date.now();
    let updateInterval = MyLibrary.psync_get_uint_value('updater-interval');
    let nextNotificationTime = MyLibrary.psync_get_uint_value('next-notify-time');

    if (argument === null || argument === undefined) {

        if (nextNotificationTime !== null && nextNotificationTime <= now) {
            nextNotificationTime = now + updateInterval;
            MyLibrary.psync_set_uint_value('next-notify-time', nextNotificationTime);
        }
    } else {
        if (updateInterval !== argument) {
            MyLibrary.psync_set_uint_value('updater-interval', argument);
        }
        if (nextNotificationTime !== null) {
            if (argument === neverRemindForUpdate) {
                MyLibrary.psync_set_uint_value('next-notify-time', argument);
            } else {
                nextNotificationTime = now + argument;
                MyLibrary.psync_set_uint_value('next-notify-time', nextNotificationTime);

            }

        }
    }
}

var getNextNotificationTime = function () {
    return MyLibrary.psync_get_uint_value('next-notify-time');
}

var updateAlreadyDownloaded = function () {
    if (newUpdateDownloaded) {
        return true;
    } else {
        return false;
    }
}

var getUpdaterInterval = function () {
    let updateInterval = MyLibrary.psync_get_uint_value('updater-interval');
    switch (updateInterval) {
        case oneHourInMilisecs:
            return 'After One Hour';
        case sixHoursInMilisecs:
            return 'After Six Hours';
        case dayInMilisecs:
            return 'Tomorrow';
        case oneWeekInMilisecs:
            return 'Next Week';
        case neverRemindForUpdate:
            return 'Never';
        default:
            return 'Error'
    }
}

var initiateUpdater = function () {

    updater.on('checking-for-update', function () {
        // log.info('CHECKING FOR UPDATE');
        // console.log('CHECKING FOR UPDATE');
    });

    updater.on('update-available', (meta) => {

        if (manualUpdateCheckRequested) {
            manualUpdateCheckRequested = false;
            choiceDialogForAvailableUpdate();
        }

        // console.log('UPDATE AVAILABLE');
        // log.info('UPDATE AVAILABLE');
        // log.info('ETO READMETO - ' + meta);
        // log.info('APP VERSION - ' + app.getVersion());

        releaseNotesOfNewVersion = meta.readme;
        updateVersionNumber = meta.version;

        if (appWindow !== null && !appWindow.isDestroyed() && currentOpenTab === 'about') {
            appWindow.webContents.send('update-available');
        }
        // updater.downloadUpdate();
    });

    updater.on('update-not-available', () => {
        // log.info('UPDATE NOT AVAILABLE');

        if (manualUpdateCheckRequested) {
            manualUpdateCheckRequested = false;
            showMessage('Latest version already installed!');
        }
    });

    updater.on('update-downloading', (meta) => {
        isDownloadingInProgress = true;
        if (appWindow !== null && !appWindow.isDestroyed() && currentOpenTab === 'about') {
            appWindow.webContents.send('update-downloading');
        }

        //TODO Animation for download/install activityx
        // log.info('UPDATE DOWNLOADING');
        // log.info(meta);
    });

    updater.on('update-downloaded', (meta) => {
        isDownloadingInProgress = false;
        if (appWindow !== null && !appWindow.isDestroyed() && currentOpenTab === 'about') {
            appWindow.webContents.send('update-downloaded');
        }

        // log.info('UPDATE DOWNLOADED');
        // log.info(meta);
        //TODO removeUpdateDownloadingIndicator();
        updateDownloadedAskForRestart();
    });

    updater.on('error', (err) => {
        log.info(err);
    });

    updater.init({
        url: 'https://api.pcloud.com/getlastversion?os=ELECTRON',
        autoDownload: false,
        checkUpdateOnStart: false,
        // logger: log
    });
    // updater.checkForUpdates();
    // updater.setFeedURL('https://api73.pcloud.com/getlastversion?os=ELECTRON');
    // updater.setOptions('url', 'https://api73.pcloud.com/getlastversion?os=ELECTRON')
    // updater.setOptions('logger', log);
    // updater.setOptions('checkUpdateOnStart', false);
    // updater.setOptions('autoDownload', false);
    // updater.checkForUpdates();
    // updater.init('https://api73.pcloud.com/getlastversion?os=ELECTRON');
};

var isDownloadingInProgressFunc = function () {
    return isDownloadingInProgress;
}

var checkForUpdates = function () {
    let updateInterval = MyLibrary.psync_get_uint_value('updater-interval');
    updater.checkForUpdates();
}

var manualCheckForUpdate = function () {
    manualUpdateCheckRequested = true;
    updater.checkForUpdates();
}

var getReleaseNotes = function () {
    if (releaseNotesOfNewVersion === "") {
        return "Latest version is already installed!"
    } else {
        return releaseNotesOfNewVersion;
    }
}

var getUpdateVersion = function () {
    if (updateVersionNumber === "") {
        return "";
    } else {
        return updateVersionNumber;
    }
}

var newVersionChecker = function () {
    let updateInterval = MyLibrary.psync_get_uint_value('updater-interval');
    if (updateVersionNumber !== "" && updateVersionNumber !== "1.7.9" && updateVersionNumber !== app.getVersion()) {
        return true;
        // console.log("New version -> true");
    } else {
        // console.log("New version -> false");
        return false;
    }
}

var initiateUpdate = function () {
    updater.downloadUpdate();
}

var openDialogError = function () {
    dialog.showMessageBox(loginWindow,
        {
            type: 'error',
            title: 'Error',
            buttons: ['Close'],
            message: 'Wrong username or password'
        });
};

function InitialLoadOpenCryptoPage() {
    if (!appWindow || appWindow.isDestroyed()) {
        openApp('#/crypto');
    }
    else {
        appWindow.webContents.executeJavaScript(
            'document.getElementById("crypto").click(); ' +
            'setActive("crypto");');
        appWindow.focus();
    }
}
function getCryptoHint() {

    let buf = new Buffer(256);
    let ptr = buf.ref();
    let hintResult = MyLibrary.psync_crypto_get_hint(ptr);
    let hintString = ref.deref(ptr).toString('utf-8');
    let terminatingNullPos = hintString.indexOf('\u0000');
    if (terminatingNullPos >= 0) {
        hintString = hintString.substr(0, terminatingNullPos);
    }
    return hintString;
}

function checkCryptoStatus() {
    let isCryptoStarted = MyLibrary.psync_crypto_isstarted();
    // console.log("isCryptoStarted is -> " + isCryptoStarted);

    let isCryptoSetup = MyLibrary.psync_crypto_issetup();
    // console.log("isCryptoSetup is -> " + isCryptoSetup);

    let cryptoSubscription = MyLibrary.psync_crypto_hassubscription();
    // console.log("cryptoSubscription is -> " + cryptoSubscription);

    let cryptoIsExpired = MyLibrary.psync_crypto_isexpired();
    // console.log("cryptoIsExpired is -> " + cryptoIsExpired);

    // let cryptoExpiresIn = MyLibrary.psync_crypto_expires();
    // console.log("cryptoExpiresIn is -> " + cryptoExpiresIn);

    if (cryptoIsExpired === 1) {
        return 1;
    } else if (cryptoIsExpired === 0 && isCryptoSetup === 0 && cryptoSubscription === 1) {
        return 2;
    } else if (cryptoIsExpired === 0 && isCryptoSetup === 1 && cryptoSubscription === 1) {
        return 3;
    } else if (cryptoIsExpired === 0 && isCryptoSetup === 1 && cryptoSubscription === 0) {
        return 4;
    } else {
        return 0;
    }
}

//Instantiate all browserWindows as null so they are not garbage collected
var popupWindow = null;
var registerWindow = null;
var inviteFolderWindow = null;
var virtualFolderWindow = null;
var changePasswordWindow = null;
var unlockCryptoWindow = null;
var accountRegisterConfirmation = null;
var provideEmailWindow = null;
var hiddenWindow = null;
var loginWindow = null;
var appWindow = null;
var addSyncWindow = null;
var backgroundWindow = null;
var trayIcon = null;

//Called if the user try to run the app again while running
app.makeSingleInstance((commandLine, workingDirectory) => {
    // Someone tried to run a second instance, we should focus our window.
    if (loginWindow === null || loginWindow.isDestroyed()) {
        if (appWindow !== null) {

            if (!appWindow.isDestroyed() && appWindow.isMinimized()) {
                appWindow.restore();
            } else if (appWindow.isDestroyed()) {
                appWindow = new BrowserWindow({
                    width: 888,
                    height: 500,
                    resizable: false,
                    icon: path.join(__dirname, '/app/img/icon.png')
                });
                appWindow.setMenu(null);
                appWindow.loadURL('file://' + __dirname + '/app/index.html' + "#/account");
            }
        } else if (loginWindow !== null && loginWindow.isDestroyed()) {
            appWindow = new BrowserWindow({
                width: 888,
                height: 500,
                resizable: false,
                icon: path.join(__dirname, '/app/img/icon.png')
            });
            appWindow.setMenu(null);
            appWindow.loadURL('file://' + __dirname + '/app/index.html' + "#/account");
            appWindow.webContents.executeJavaScript('setActive("account");');
        }
    }

})

// if (shouldQuit) {
//     app.quit()
// }

//Initialized when the app is 'ready'
app.on('ready', function () {

    //The app will stop when all browser windows are closed
    //The hiddenWindow prevents the app from closing
    log.transports.file.level = 'silly';
    log.appName = 'pcloud';

    hiddenWindow = new BrowserWindow({ show: false });
    MyLibrary.init();
    MyLibrary.set_status_callback(callback);
    MyLibrary.set_event_callback(pevent_callback);
    let appImagePath = process.env.APPIMAGE;
    pcloudAutoLauncher = new AutoLaunch({
        name: 'pcloud',
        path: appImagePath,
    });

    eNotify = require('electron-notify');
    setNotificationOptions();
    configUpdaterSettings();
    initiateUpdater();
    createBackgroundWorker();
    checkForUpdates();
});

//create the Tray menu
var initTray = function () {
    //initializing the tray menu
    trayIcon = new Tray(path.join(__dirname, 'app/img/icon-pcloud.png'));

    //setting the menu options
    var trayMenuTemplate = [
        {
            label: 'Open Drive',
            click: function () {
                shell.showItemInFolder(process.env['HOME'] + '/pCloudDrive/./');
            }
        },
        {
            label: 'Account',
            click: function () {
                if (!appWindow || appWindow.isDestroyed()) {
                    openApp('#/account');
                }
                else {
                    appWindow.webContents.executeJavaScript(
                        'document.getElementById("account").click(); ' +
                        'setActive("account");');
                    appWindow.show();
                }
            }
        },
        {
            label: 'Sync',
            click: function () {
                if (!appWindow || appWindow.isDestroyed()) {
                    openApp('#/sync');
                }
                else {
                    appWindow.webContents.executeJavaScript(
                        'document.getElementById("sync").click(); ' +
                        'setActive("sync");');
                    appWindow.show();
                }
            }
        },
        {
            label: 'Shares',
            click: function () {
                if (!appWindow || appWindow.isDestroyed()) {
                    openApp('#/shares');
                }
                else {
                    appWindow.webContents.executeJavaScript(
                        'document.getElementById("shares").click(); ' +
                        'setActive("shares");');
                    appWindow.show();
                }
            }
        },
        {
            label: 'Crypto',
            click: function () {
                if (!appWindow || appWindow.isDestroyed()) {
                    openApp('#/crypto');
                }
                else {
                    appWindow.webContents.executeJavaScript(
                        'document.getElementById("crypto").click(); ' +
                        'setActive("crypto");');
                    appWindow.show();
                }
            }
        },
        {
            label: 'Help',
            click: function () {
                if (!appWindow || appWindow.isDestroyed()) {
                    openApp('#/help');
                }
                else {
                    appWindow.webContents.executeJavaScript(
                        'document.getElementById("help").click(); ' +
                        'setActive("help");');
                    appWindow.show();
                }
            }
        },
        {
            label: 'About',
            click: function () {
                if (!appWindow || appWindow.isDestroyed()) {
                    openApp('#/about');
                }
                else {
                    appWindow.webContents.executeJavaScript(
                        'document.getElementById("about").click(); ' +
                        'setActive("about");');
                    appWindow.show();
                }
            }
        },
        {
            label: 'Exit',
            click: function () {
                if (isAppReady) {
                    // fs.remove(process.env['HOME'] + '/Desktop/openApp', function (err) {
                    //     if (err) return console.error(err);

                    //     console.log("success!")
                    // });
                    isAppReady = false;
                    app.quit();
                    // setTimeout(function () {
                    //     MyLibrary.finalize();
                    // }, 500);
                }
                else {
                    app.quit();
                }
            }
        }
    ];
    //making the menu
    var trayMenu = null;
    trayMenu = Menu.buildFromTemplate(trayMenuTemplate);
    trayIcon.setContextMenu(trayMenu);
    trayIcon.on('click', () => {
        shell.showItemInFolder(process.env['HOME'] + '/pCloudDrive/./');
    });
};

//ensuring that finalize is called before quiting the app
app.on('before-quit', function () {
    console.log('Bye Bye!');
    if (isAppReady !== null && isAppReady === true) {
        isAppReady = false;
    }
    isAppReady = false;
    MyLibrary.finalize();
});

//creating a new window
var openLoginWindow = function () {
    unlinked = false;
    if (loginWindow == null || loginWindow.isDestroyed()) {
        if (registerWindow == null || registerWindow.isDestroyed()) {

            loginWindow = new BrowserWindow({
                height: 475,
                width: 525,
                resizable: false,
                icon: path.join(__dirname, '/app/img/icon.png')
            });
            loginWindow.setMenu(null);
            //loading the html file
            loginWindow.loadURL('file://' + __dirname + '/app/startscreen.html');
            //use Chrome's developer tools
            // loginWindow.webContents.openDevTools({ detach: true });

            loginWindow.on("close", function (e) {
                // loginWindow.close();
                if (!isAppReady) {
                    var choice = require('electron').dialog.showMessageBox(this,
                        {
                            type: 'question',
                            buttons: ['Yes', 'No'],
                            title: 'Confirm',
                            message: 'Are you sure you want to quit?'
                        });
                    if (choice == 1) {
                        e.preventDefault();
                    }
                    else {
                        app.quit();
                    }
                }
            });
        }
    }
};

//appWindow
var openApp = function (partial) {
    appWindow = new BrowserWindow({
        width: 888,
        height: 500,
        resizable: false,
        icon: path.join(__dirname, '/app/img/icon.png')
    });
    appWindow.setMenu(null);
    appWindow.loadURL('file://' + __dirname + '/app/index.html' + partial);
    // appWindow.webContents.openDevTools({ detach: true });
    var id = partial.substr(2);
    appWindow.webContents.executeJavaScript('setActive("' + id + '");');

};

ipcMain.on('login-screen-unlink-pressed', function () {
    let currentWindow = BrowserWindow.getFocusedWindow();
    var choice = require('electron').dialog.showMessageBox(currentWindow,
        {
            type: 'question',
            buttons: ['Yes', 'No'],
            title: 'Confirm',
            message: 'By unlinking you will lose all pCloud settings and Sync configurations for the current account on this computer. Are you sure?'
        });
    if (choice == 0) {
        MyLibrary.unlinklib();
        if (isAppReady) {
            isAppReady = false;
        }

        loginWindow.destroy();
        openLoginWindow();
    } else {
        loginWindow.reload();
    }

});

var unlinkPressed = function () {
    unlinked = true;
    if (!appWindow.isDestroyed()) {
        appWindow.destroy();
        MyLibrary.unlinklib();
        backgroundWindow.destroy();
        trayIcon.destroy()
        if (isAppReady) {
            isAppReady = false;
        }
    }
}

ipcMain.on('unlink-pressed', function () {

    let currentWindow = BrowserWindow.getFocusedWindow();
    var choice = require('electron').dialog.showMessageBox(currentWindow,
        {
            type: 'question',
            buttons: ['Yes', 'No'],
            title: 'Confirm',
            message: 'By unlinking you will lose all pCloud settings and Sync configurations for the current account on this computer. Are you sure?'
        });
    if (choice == 0) {
        appWindow.webContents.send('start-spinner');
        unlinkPressed();
    }
});

ipcMain.on('logout-pressed', function logoutPressed() {
    // MyLibrary.psync_fs_stop();
    unlinked = true;
    appWindow.destroy();
    backgroundWindow.destroy();
    trayIcon.destroy()
    MyLibrary.psync_set_string_value('fb_access_token', "");
    MyLibrary.logout();
    if (isAppReady) {
        isAppReady = false;
    }
});

// Check if the user is business
function isUserBusiness() {
    // const BUSINESS = 'business';
    // var businessStringToArrayValue = Buffer.from(BUSINESS + "\0");
    var result = MyLibrary.psync_get_bool_value('business');
    if (result == 1) {
        return true;
    } else {
        return false;
    }
}

function getCompanyName() {
    const COMPANY = 'company';
    var companyStringToArrayValue = Buffer.from(COMPANY + "\0");
    var result = MyLibrary.psync_get_string_value(companyStringToArrayValue);
    if (result != null) {
        return result;
    } else {
        return "";
    }
}

// Return if the user is premium or not
function getIfUserIsPremium() {
    // const PREMIUM = 'premium';
    // var userPremiumStringToArrayValue = Buffer.from(PREMIUM);
    var result = MyLibrary.psync_get_bool_value('premium');
    if (result == 0) {
        return false;
    } else {
        return true;
    }
}

// Return the used quota on that account
function getUsedQuota() {
    // const USED_QUOTA_STRING = 'usedquota';
    // var usedQuotaStringToArrayValue = Buffer.from(USED_QUOTA_STRING + "\0");
    var usedQuotaInBytes = MyLibrary.psync_get_uint_value('usedquota');
    // var usedQuotaInGB = (usedQuotaInBytes / Math.pow(1024, 3)).toFixed(2);
    return usedQuotaInBytes;
}

// Return the total quota on that account
function getTotalQuota() {
    // const TOTAL_QUOTA_STRING = 'quota';
    // var totalQuotaStringToArrayValue = Buffer.from(TOTAL_QUOTA_STRING + "\0");
    var totalQuotaInBytes = MyLibrary.psync_get_uint_value('quota');
    // var totalQuotaInGB = (totalQuotaInBytes / Math.pow(1024, 3)).toFixed(2);
    return totalQuotaInBytes;
}

function getAuthToken() {
    var authToken = MyLibrary.get_token();
    return authToken;
}

function getUserName() {
    var userName = MyLibrary.psync_get_username();
    return userName;
}

//log-in channel
ipcMain.on('log-in', function (event, args) {
    isOnline().then(online => {
        if (online) {
            sendTokenInfo()
            var username = Buffer.from(args[0] + "\0");
            var password = Buffer.from(args[1] + "\0");
            var save = args[2] ? 1 : 0;

            var result = MyLibrary.login(username, password, save);

        } else {
            showError('No internet connection!');
        }
    });
});

function showError(textMessage) {

    let currentWindow = findActiveWindow();

    if (currentWindow !== null) {
        dialog.showMessageBox(currentWindow,
            {
                type: 'error',
                title: 'Error',
                buttons: ['Close'],
                message: '' + textMessage
            });
    }
}

function showMessage(textMessage) {
    // var currentWindow = BrowserWindow.getFocusedWindow();

    let currentWindow = findActiveWindow();

    if (currentWindow !== null) {
        dialog.showMessageBox(currentWindow,
            {
                type: 'info',
                title: 'Information',
                buttons: ['Close'],
                message: '' + textMessage
            });
    }
}

function cryptoStarted() {
    let checkIfCryptoIsStarted = MyLibrary.psync_crypto_isstarted();
    if (checkIfCryptoIsStarted === 1) {
        return true;
    } else {
        return false;
    }
}

function cryptoSetup(cryptoPass, hintString) {
    let hint = "";
    if (hintString != undefined) {
        hint = hintString;
    }
    let cryptoSetupResult = MyLibrary.psync_crypto_setup(cryptoPass, hint);

    let crypto_pass = Buffer.from(cryptoPass + "\0");
    let cryptoStartResult = MyLibrary.statrt_crypto(crypto_pass);

    let buf1 = new Buffer(256);
    let errPointer = buf1.ref();
    let buf2 = new Buffer(256);
    let newFolderIdPointer = buf2.ref();

    let mkFolderResult = MyLibrary.psync_crypto_mkdir(0, 'Crypto Folder', errPointer, newFolderIdPointer);

    // let newCryptoFolderId = ref.readUInt64BE(newFolderIdPointer, 0);
    // console.log(newCryptoFolderId);

    if (cryptoSetupResult !== 0) {
        let mkFolderErrorMessage = ref.deref(errPointer).toString('utf-8');
        let terminatingNullPos = mkFolderErrorMessage.indexOf('\u0000');
        if (terminatingNullPos >= 0) {
            mkFolderErrorMessage = mkFolderErrorMessage.substr(0, terminatingNullPos);
        }
        showError(mkFolderErrorMessage);
        return false;
    } else {
        if (cryptoSetupResult === 0) {
            return true;
        } else {
            showError("Could not setyp crypto. Error - " + cryptoSetupResult);
            return false;
        }
    }
}

function cryptoPassDifficulty10000(password) {
    if (password === undefined) {
        return 0;
    }

    let result = MyLibrary.psync_password_quality10000(password);

    if (result < 2999) {
        return 1;
    } else if (result < 5999) {
        return 2;
    } else if (result < 8999) {
        return 3;
    } else if (result < 11999) {
        return 4;
    } else if (result < 14999) {
        return 5;
    } else if (result < 17999) {
        return 6;
    } else if (result < 20999) {
        return 7;
    } else if (result < 23999) {
        return 8;
    } else if (result < 27999) {
        return 9;
    } else if (result >= 29999) {
        return 10;
    }
}

function cryptoPassMemoryConfirmation() {
    var choice = require('electron').dialog.showMessageBox(appWindow,
        {
            type: 'question',
            buttons: ['I got it', 'Cancel'],
            title: 'Confirm Crypto Pass',
            message: 'Did you memorize your Crypto Pass? Once forgotten it cannot be restored.'
        });
    if (choice === 0) {
        return true;
    } else {
        return false;
    }
}

function resetCryptoPassConfirmation() {
    var choice = require('electron').dialog.showMessageBox(appWindow,
        {
            type: 'question',
            buttons: ['Yes', 'No'],
            title: 'Confirm Reset',
            message: 'By resetting your Crypto Pass you will delete all currently encrypted files. Are you sure?'
        });
    if (choice === 0) {
        showMessage("An email with instructions has been sent to " + getUserName() + " .Please, follow the steps to reset your Crypto Pass.");
        return true;
    } else {
        return false;
    }
}

function resetCryptoPass() {
    let result = MyLibrary.psync_crypto_reset();
    if (result === 0) {
        return true;
    } else {
        showError("Error reseting crypto pass");
        return false;
    }
}

ipcMain.on('show-hint', function () {
    showMessage("Your hint is: " + getCryptoHint());
});

function getCryptoExpirationDaysString() {
    let cryptoExpireTime = MyLibrary.psync_crypto_expires() * 1000;
    if (cryptoExpireTime === 0) {
        showError("Crypto not setup!");
        return "Crypto not setup!";
    }

    let now = Date.now();
    let diff = cryptoExpireTime - now;
    let result = diff / (1000 * 60 * 60 * 24);

    if (diff < 0) {
        showError("Crypto Expired");
        return "Crypto Expired"
    } else {
        result = Math.ceil(result);
        return "Crypto expires in " + result + " days";
    }
}

function cryptoExpiredGracePeriod() {
    let cryptoExpireTime = MyLibrary.psync_crypto_expires() * 1000;

    if (cryptoExpireTime === 0) {
        return false;
    }

    let now = Date.now();
    let diff = cryptoExpireTime - now;
    let result = diff / (1000 * 60 * 60 * 24);
    if (result < 0 && result > -30) {
        return true;
    } else {
        return false;
    }
}

function unlockCrypto(cryptoPass) {
    var crypto_pass = Buffer.from(cryptoPass + "\0");
    var startResult = MyLibrary.statrt_crypto(crypto_pass);
    switch (startResult) {
        case 0:
            shell.showItemInFolder(process.env['HOME'] + '/pCloudDrive/Crypto Folder/./');
            if (appWindow != null && !appWindow.isDestroyed()) {
                appWindow.webContents.send('crypto-started');
            } else {
                unlockCryptoWindow.close();
            }
            return true;
        case 6:
            showError('Incorrect Crypto Pass!');
            return false;
        default:
            return false;
    }
}

function lockCryptoFolder() {
    if (cryptoStarted()) {
        // let result = MyLibrary.stop_crypto();
        let result = MyLibrary.psync_crypto_stop();
        if (result === 0) {
            return true;
        } else {
            return false;
        }
    } else {
        showError("Crypto Error - Crypto is stopped");
        return false;
    }

}

//select directory
function selectDirectory(isRemote) {
    return dialog.showOpenDialog(addSyncWindow, {
        defaultPath: process.env['HOME'] + (isRemote ? '/pCloudDrive/' : ''),
        properties: ['openDirectory']
    });
};

function selectCachePath() {
    return dialog.showOpenDialog(appWindow, {
        defaultPath: process.env['HOME'],
        properties: ['openDirectory']
    });
};

//export for use from renderer process
exports.selectDirectory = function (isRemote) {
    // dialog.showOpenDialog as before
    return selectDirectory(isRemote);
};

//add new sync function
function addNewSyncWindow() {
    addSyncWindow = new BrowserWindow({
        height: 250,
        width: 450,
        resizable: false,
        parent: appWindow,
        modal: true
    });
    addSyncWindow.setMenu(null);
    addSyncWindow.loadURL('file://' + __dirname + '/app/addsync.html');
    // addSyncWindow.webContents.openDevTools({ detach: true });
}

//export for use from renderer process
exports.addNewSyncWindow = function () {
    if (!addSyncWindow || addSyncWindow.isDestroyed()) {
        addNewSyncWindow();
    }
    else {
        addSyncWindow.focus();
    }
};

//export for use from renderer process
exports.drivePath = function () {
    return process.env['HOME'] + '/pCloudDrive/';
};

//add sync channel
ipcMain.on('add-sync', function (event, args) {
    var synctype = parseInt(3); // Adding "3" for bothways sync
    var local = args[0].toString() + '/';
    var remote = args[1].toString();
    remote = remote.substring(process.env['HOME'].length + 12);
    //converting to suitable types
    var localpath = Buffer.from(local + "\0");
    var remotepath = Buffer.from(remote + "\0");
    // console.log(local);
    // console.log(remote);
    // console.log('synctype is' + args[2]);
    console.log('syncid is' + MyLibrary.psync_add_sync_by_path(localpath, remotepath, synctype));
    //appWindow.reload();
    appWindow.webContents.send('refresh-sync-page');
    addSyncWindow.close();
});

//delete sync channel
ipcMain.on('delete-sync', function (event, syncid) {
    let result = MyLibrary.psync_delete_sync(syncid);
    if (result === 0) {
        console.log("Sync deleted!");
    }
});

//closing the addSync window
ipcMain.on('close-add-sync-window', function () {
    addSyncWindow.close();
});

ipcMain.on('sign-up', function () {

    let currentUser = getUserName();
    if (currentUser == null || currentUser == "") {
        createRegisterWindow();
    } else {
        showMessage('The account "' + currentUser + '" is currently linked to this computer. You need to unlink in order to register a new account.');
    }

});

ipcMain.on('change-password', function () {
    createChangePasswordWindow();
});

ipcMain.on('unlock-crypto-window', function () {
    createUnlockCryptoWindow();
});

ipcMain.on('unlock-crypto-window-close', function () {
    unlockCryptoWindow.close();
});

ipcMain.on('verify-email', function () {
    isOnline().then(online => {
        if (online) {

            let buf = new Buffer(256);
            let ptr = buf.ref();
            let result = MyLibrary.psync_verify_email(ptr);

            if (result != 0) {
                let message = ref.deref(ptr).toString('utf-8');
                let terminatingNullPos = message.indexOf('\u0000');

                if (terminatingNullPos >= 0) {
                    message = message.substr(0, terminatingNullPos);
                }
                showError(message);
            } else {
                let currentUser = getUserName();
                let message = "An email with a verification link has been sent to " + currentUser + ". Please, check your email."
                showMessage(message);
            }

        } else {
            showError('No internet connection!');
        }
    });
});

ipcMain.on('submit-register', function (event, args) {
    isOnline().then(online => {
        if (online) {

            var username = Buffer.from(args[0] + "\0");
            var password = Buffer.from(args[1] + "\0");
            var agreement = args[2] ? 1 : 0;

            let buf = new Buffer(256);
            let ptr = buf.ref();
            let result = MyLibrary.psync_register(username, password, agreement, ptr);
            if (result != 0) {
                let message = ref.deref(ptr).toString('utf-8');
                let terminatingNullPos = message.indexOf('\u0000');

                if (terminatingNullPos >= 0) {
                    message = message.substr(0, terminatingNullPos);
                }
                showError(message);
            } else {
                let loginResult = MyLibrary.login(username, password, 0);
            }

        } else {
            showError('No internet connection!');
        }
    });
});

var createRegisterWindow = function () {
    if (loginWindow != null || !loginWindow.isDestroyed()) {
        loginWindow.destroy();
    }

    if (registerWindow == null || registerWindow.isDestroyed()) {
        registerWindow = new BrowserWindow({
            height: 475,
            width: 525,
            resizable: false,
            icon: path.join(__dirname, '/app/img/icon.png')
        });
        registerWindow.setMenu(null);
        //loading the html file
        registerWindow.loadURL('file://' + __dirname + '/app/registerwindow.html');


        registerWindow.on("close", function (e) {
            var choice = require('electron').dialog.showMessageBox(this,
                {
                    type: 'question',
                    buttons: ['Yes', 'No'],
                    title: 'Confirm',
                    message: 'Are you sure you want to quit?'
                });
            if (choice == 1) {
                e.preventDefault();
            }
            else {
                app.quit();
            }
        });
        // registerWindow.webContents.openDevTools({ detach: true });
    }
}

var createChangePasswordWindow = function () {

    changePasswordWindow = new BrowserWindow({
        height: 275,
        width: 275,
        resizable: false,
        modal: true,
        parent: appWindow,
        icon: path.join(__dirname, '/app/img/icon.png')
    });

    changePasswordWindow.setMenu(null);

    //loading the html file
    changePasswordWindow.loadURL('file://' + __dirname + '/app/change-password-window.html');

    // changePasswordWindow.webContents.openDevTools({ detach: true });
}

function userHasActiveCrypto() {
    let cryptoStatus = checkCryptoStatus();
    if (cryptoStatus === 3 || cryptoStatus === 4) {
        return true;
    } else if (cryptoStatus === 1 && cryptoExpiredGracePeriod()) {
        return true;
    } else {
        return false;
    }
}

function checkAndSuggestUnlockCrypto() {

    let unlockOnStart = getBoolValue("unlockcryptoonstart");
    if (unlockOnStart === true && userHasActiveCrypto()) {
        createUnlockCryptoWindow();
    }
}

var createUnlockCryptoWindow = function () {

    if (!unlockCryptoWindow || unlockCryptoWindow.isDestroyed()) {
        unlockCryptoWindow = new BrowserWindow({
            height: 155,
            width: 275,
            resizable: false,
            modal: true,
            parent: appWindow,
            icon: path.join(__dirname, '/app/img/icon.png')
        });

        unlockCryptoWindow.setMenu(null);

        //loading the html file
        unlockCryptoWindow.loadURL('file://' + __dirname + '/app/unlock-crypto-window.html');

        // unlockCryptoWindow.webContents.openDevTools({ detach: true });
    } else {
        unlockCryptoWindow.focus();
    }
}

var createRegisterConfirmationWindow = function () {
    accountRegisterConfirmation = new BrowserWindow({
        height: 475,
        width: 525,
        resizable: false,
        modal: true,
        parent: loginWindow,
        icon: path.join(__dirname, '/app/img/icon.png')
    });

    accountRegisterConfirmation.setMenu(null);

    // accountRegisterConfirmation.webContents.openDevTools({ detach: true });

    //loading the html file
    accountRegisterConfirmation.loadURL('file://' + __dirname + '/app/register-confirmation-window.html');

    accountRegisterConfirmation.on("close", function (e) {
        MyLibrary.psync_set_string_value('fb_access_token', "");
        MyLibrary.psync_set_string_value('username', "");
    });
}

ipcMain.on('fb-confirm-register', function () {
    let userName = getUserName();
    let fbAccessTokenToArrayValue = Buffer.from('fb_access_token' + "\0");
    let fbAcessToken = MyLibrary.psync_get_string_value(fbAccessTokenToArrayValue);

    let buf = new Buffer(256);
    let ptr = buf.ref();

    var result = MyLibrary.fb_login_register(userName, fbAcessToken, 1, ptr)

    let message = ref.deref(ptr).toString('utf-8');
    let terminatingNullPos = message.indexOf('\u0000');

    if (terminatingNullPos >= 0) {
        message = message.substr(0, terminatingNullPos);
    }
});

var createProvideEmailWindow = function () {
    provideEmailWindow = new BrowserWindow({
        height: 475,
        width: 525,
        resizable: false,
        modal: true,
        parent: loginWindow,
        icon: path.join(__dirname, '/app/img/icon.png')
    });

    provideEmailWindow.setMenu(null);

    // provideEmailWindow.webContents.openDevTools({ detach: true });

    //loading the html file
    provideEmailWindow.loadURL('file://' + __dirname + '/app/provide-email-window.html');
}

ipcMain.on('email-provided', function (event, args) {
    // MyLibrary.psync_set_string_value('username', args);
    // let userName = getUserName();
    let fbAccessTokenToArrayValue = Buffer.from('fb_access_token' + "\0");
    let fbAcessToken = MyLibrary.psync_get_string_value(fbAccessTokenToArrayValue);

    let buf = new Buffer(256);
    let ptr = buf.ref();

    var result = MyLibrary.fb_login_register(args, fbAcessToken, 1, ptr)
    let message = ref.deref(ptr).toString('utf-8');
    let terminatingNullPos = message.indexOf('\u0000');

    if (terminatingNullPos >= 0) {
        message = message.substr(0, terminatingNullPos);
    }

    if (result == 1046) {
        var result = MyLibrary.fb_authorize(fbCallback)
        shell.openExternal(result);
    } else if (result != 0) {
        showError(message);
    }
});

ipcMain.on('return-to-login', function () {
    if (registerWindow != null && !registerWindow.isDestroyed()) {
        registerWindow.destroy();
    }
    openLoginWindow();
});

ipcMain.on('close-current-window', function () {
    let currentWindow = BrowserWindow.getFocusedWindow();
    currentWindow.close();
});

ipcMain.on('refresh-screen', function () {
    let result = findActiveWindow();
    if (result.getTitle() === "pCloud Drive") {
        result.reload();
    }
});

ipcMain.on('refresh-appWindowShares-openSpecificTab', function (event, args) {
    setTimeout(function () {
        appWindow.reload();
        setTimeout(function () {
            appWindow.webContents.send('set-active-tab', args);
        }, 750)
    }, 1500);
});

ipcMain.on('forgot-password', function (event, args) {
    isOnline().then(online => {
        if (online) {
            let buf = new Buffer(256);
            let ptr = buf.ref();
            let stringValueToPass;
            if (args != null && args != undefined && args != "") {
                let userName = args;
                stringValueToPass = Buffer.from(userName + "\0");
            } else {
                let userName = getUserName();
                stringValueToPass = Buffer.from(userName + "\0");
            }

            let result = MyLibrary.psync_lost_password(stringValueToPass, ptr);

            if (result != 0) {
                let message = ref.deref(ptr).toString('utf-8');
                let terminatingNullPos = message.indexOf('\u0000');

                if (terminatingNullPos >= 0) {
                    message = message.substr(0, terminatingNullPos);
                }
                showError(message);
            } else {
                let message = "An email with the password reset instructions was sent to your email."
                showMessage(message);
            }

        } else {
            showError('No internet connection!');
        }
    });
});

ipcMain.on('submit-change-password', function (event, args) {
    isOnline().then(online => {
        if (online) {

            var currentPass = Buffer.from(args[0] + "\0");
            var newPass = Buffer.from(args[1] + "\0");

            let buf = new Buffer(256);
            let ptr = buf.ref();
            let result = MyLibrary.psync_change_password(currentPass, newPass, ptr);

            if (result != 0) {
                let message = ref.deref(ptr).toString('utf-8');
                let terminatingNullPos = message.indexOf('\u0000');

                if (terminatingNullPos >= 0) {
                    message = message.substr(0, terminatingNullPos);
                }
                showError(message);
            } else {
                showMessage('Password changed successfully')
                changePasswordWindow.close();
            }

        } else {
            showError('No internet connection!');
        }
    });
});

ipcMain.on('close-change-password-window', function () {
    changePasswordWindow.close();
});

// FACEBOOK INTEGRATION

ipcMain.on('continue-with-facebook', function () {

    let userName = getUserName();
    let fbAccessTokenToArrayValue = Buffer.from('fb_access_token' + "\0");
    let fbAcessToken = MyLibrary.psync_get_string_value(fbAccessTokenToArrayValue);
    fbAccessTokenToArrayValue = Buffer.from(fbAcessToken + "\0");
    if (fbAcessToken != null && fbAcessToken !== "") {

        if (userName != null && userName !== "") {
            sendTokenInfo()
            let buf = new Buffer(256);
            let ptr = buf.ref();
            var result = MyLibrary.fb_login(fbAccessTokenToArrayValue, ptr);
            // console.log('result = MyLibrary.fb_login(fbAccessTokenToArrayValue, ptr) = ' + result);
            if (result == 2037) {
                // new account needs to be created. Open the register window
                createRegisterConfirmationWindow();

            } else if (result == 1046) {
                showError('This Facebook account is not linked to your current pCloud account. You can link both accounts in Settings on my.pcloud.com or Unlink to continue with this Facebook account.');
            } else if (result != 0) {
                /*Result 2018 - Invalid 'mail' provided. Facebook returns invalid email. We should ask the user to provide email and 
                                    try to login manually using fb_login_register. Token parameter contains acquired facebook access token.*/

                /*Result 2038 - User with this email is already registered. We should ask the user to provide another email and 
                                    try to login manually using fb_login_register. Token parameter contains acquired facebook access token.*/

                let message = ref.deref(ptr).toString('utf-8');
                let terminatingNullPos = message.indexOf('\u0000');

                if (terminatingNullPos >= 0) {
                    message = message.substr(0, terminatingNullPos);
                }
                showError(message);
            }

        } else {

            //USER HAS TOKEN BUT THE EMAIL IS NOT PROVIDED

            isOnline().then(online => {
                if (online) {

                    sendTokenInfo();
                    let buf = new Buffer(256);
                    let ptr = buf.ref();
                    var result = MyLibrary.fb_login(fbAccessTokenToArrayValue, ptr);

                    //Valid and not expired Facebook access token is required in field 'access_token'.
                    if (result = 1033) {
                        //EMAIL NEEDED
                        createProvideEmailWindow();
                    } else {
                        let message = ref.deref(ptr).toString('utf-8');
                        let terminatingNullPos = message.indexOf('\u0000');

                        if (terminatingNullPos >= 0) {
                            message = message.substr(0, terminatingNullPos);
                        }

                        MyLibrary.psync_set_string_value('username', message);
                        var testValue = getUserName();
                        // showError(message);
                    }
                }
            });
        }

    } else {
        // NO FB_ACCESS_TOKEN FOUND
        sendTokenInfo();
        var result = MyLibrary.fb_authorize(fbCallback);
        shell.openExternal(result);
    }
});

//SETTINGS SCREEN CALLS

function setBoolValue(stringKey, intValue) {
    MyLibrary.psync_set_bool_value(stringKey, intValue);
}

function getBoolValue(stringKey) {
    let result = MyLibrary.psync_get_bool_value(stringKey);
    if (result === 0) {
        return false;
    }
    return true;
}

function getStringSetting(key) {
    // 'psync_get_int_setting': ['int64', ['string']],
    let string = key;
    var stringToCharArray = Buffer.from(string + "\0");
    let value = MyLibrary.psync_get_string_value(stringToCharArray);
    return value;
}

function setStringSetting(key, value) {

    // 'psync_set_int_setting': ['int64', ['string', 'int64']],
    // MyLibrary.psync_set_int_setting(key, value);
    MyLibrary.psync_set_string_value(key, value + "");

}

// function disableAutoStart() {
//     pcloudAutoLauncher.disable();
//     setBoolValue('autostart', 0);
// }

// function enableAutoStart() {
//     pcloudAutoLauncher.enable();
//     setBoolValue('autostart', 1);
// }

function initAutostartSettings() {
    //  || getBoolValue("autostart") == true
    if (getStringSetting("auto-start-string") === null) {
        setStringSetting("auto-start-string", 2 + "");
        pcloudAutoLauncher.enable();
    }

}

function isAutoStartEnabled() {
    pcloudAutoLauncher.isEnabled()
        .then(function (isEnabled) {
            if (isEnabled) {
                return true;
            }
            return false;
        })
}

function getCurrentCacheLoc() {
    let currentCacheLocation = MyLibrary.psync_get_string_setting('fscachepath');
    return currentCacheLocation;
}

function clearCache() {
    MyLibrary.psync_fs_clean_read_cache();
}

function changeCacheLocation() {
    var choice = require('electron').dialog.showMessageBox(appWindow,
        {
            type: 'question',
            buttons: ['I understand and agree', 'Cancel'],
            title: 'Important!',
            message: 'Please note it is recommended that you choose a local hard disk as your cache location. Choosing a removable device may lead to service malfunction and data loss'
        });
    if (choice == 1) {

    }
    else {
        let userLocationChoice = selectCachePath();
        if (userLocationChoice != undefined) {

            let result = MyLibrary.psync_fs_move_cache(userLocationChoice[0]);

            if (result == 1) {
                showError('Target directory already has a read cache file in it');
            } else if (result == 2) {
                showError('Request to open a file for writing in the provided directory fails');
            } else if (result == 3) {
                showError("Drive's task queue is not empty. Putting the cache on a remote drive is generally not a good idea.");
            } else {
                appWindow.webContents.send('initiateLoading');
            }
        }
    }
}

function getAllSettings() {
    let stringAutoStart = "auto-start-string";
    var stringAutoStartToCharArray = Buffer.from(stringAutoStart + "\0");

    var initSettings = {
        upSpeed: MyLibrary.psync_get_int_setting("maxuploadspeed"),
        dwSpeed: MyLibrary.psync_get_int_setting("maxdownloadspeed"),
        lanSync: MyLibrary.psync_get_bool_setting('p2psync'),
        sleepstopcrypto: MyLibrary.psync_get_bool_setting('sleepstopcrypto'),
        minDiskSpace: MyLibrary.psync_get_uint_setting("minlocalfreespace"),
        cacheSize: MyLibrary.psync_get_uint_setting("fscachesize"),
        autostart: MyLibrary.psync_get_string_value(stringAutoStartToCharArray)
    };


    if (initSettings.upSpeed > 0) {
        initSettings.upSpeed = Math.round(initSettings.upSpeed / 1024);
    }

    if (initSettings.dwSpeed > 0) {
        initSettings.dwSpeed = Math.round(initSettings.dwSpeed / 1024);
    }


    if (initSettings.lanSync == 1) {
        initSettings.lanSync = true;
    } else {
        initSettings.lanSync = false;
    }

    if (initSettings.sleepstopcrypto === 1) {
        initSettings.sleepstopcrypto = true;
    } else {
        initSettings.sleepstopcrypto = false;
    }

    if (initSettings.autostart === "2") {
        initSettings.autostart = true;
    } else {
        initSettings.autostart = false;
    }

    return initSettings;
}

ipcMain.on('update-settings', function (event, args) {

    for (var key in args) {
        if (args.hasOwnProperty(key)) {
            // var element = args[key];
            // console.log(key + " -> " + args[key]);

            switch (key) {
                case "upSpeed":
                    if (args[key] != -1) {
                        args[key] = Math.round(args[key] * 1024);
                    }
                    MyLibrary.psync_set_int_setting("maxuploadspeed", args[key]);
                    break;
                case "dwSpeed":
                    if (args[key] != -1) {
                        args[key] = Math.round(args[key] * 1024);
                    }
                    MyLibrary.psync_set_int_setting("maxdownloadspeed", args[key]);
                    break;
                case "lanSync":
                    if (args[key] == false) {
                        MyLibrary.psync_set_bool_setting("p2psync", 0);
                    } else {
                        MyLibrary.psync_set_bool_setting("p2psync", 1);
                    }
                    break;
                case "sleepstopcrypto":
                    if (args[key] == false) {
                        MyLibrary.psync_set_bool_setting("sleepstopcrypto", 0);
                    } else {
                        MyLibrary.psync_set_bool_setting("sleepstopcrypto", 1);
                    }
                    break;
                case "minDiskSpace":
                    MyLibrary.psync_set_uint_setting("minlocalfreespace", args[key])
                    break;
                case "cacheSize":
                    MyLibrary.psync_set_uint_setting("fscachesize", args[key])
                    break;
                case "autostart":
                    if (args[key] == false) {
                        // "1" is for to disable the autostart
                        setStringSetting("auto-start-string", 1 + "");
                        // MyLibrary.psync_set_bool_value("autostart", 0);
                        pcloudAutoLauncher.disable();
                    } else {
                        // MyLibrary.psync_set_bool_value("autostart", 1);
                        setStringSetting("auto-start-string", 2 + "");
                        pcloudAutoLauncher.enable();
                    }
                    break;
                default:
                    break;
            }
        }
    }
});

function sendTokenInfo() {
    let appInfo = null;
    let release = os.release();
    let version = app.getVersion();
    let platform = os.platform();
    let arch = os.arch();

    appInfo = Buffer.from(platform + ' ' + arch + ' - ' + release + ', pCloud Electron ' + version + "\0");
    MyLibrary.psync_set_software_string(appInfo);
}


/* SHARES */

// incoming == 1, outgoing == 0
function getShareList(type) {
    let sharesRaw = MyLibrary.psync_list_shares(type);

    let sharesCount = sharesRaw.deref().sharecnt;
    // var sharesFromBuf = sharesRaw.deref().shares;
    psync_share_list_t = StructType({
        sharecnt: 'size_t',
        shares: ArrayType(psync_share, sharesCount)
    });
    initLibrary();

    sharesRaw = MyLibrary.psync_list_shares(type);
    var sharesFromBuf = sharesRaw.deref().shares;
    // console.log(sharesFromBuf[0].shareid);

    var listShares = [];

    for (var index = 0; index < sharesCount; index++) {
        var share = {
            shareid: sharesFromBuf[index].shareid,
            folderid: sharesFromBuf[index].folderid,
            created: sharesFromBuf[index].created,
            userid: sharesFromBuf[index].userid,
            fromemail: sharesFromBuf[index].fromemail,
            toemail: sharesFromBuf[index].toemail,
            sharename: sharesFromBuf[index].sharename,
            permissions: sharesFromBuf[index].permissions,
            canread: sharesFromBuf[index].canread,
            cancreate: sharesFromBuf[index].cancreate,
            canmodify: sharesFromBuf[index].canmodify,
            candelete: sharesFromBuf[index].candelete,
            canmanage: sharesFromBuf[index].canmanage,
            isba: sharesFromBuf[index].isba,
            isteam: sharesFromBuf[index].isteam
        };
        listShares.push(share);
    }
    return listShares;
}

function getShareRequestList(type) {
    var sharesRaw = MyLibrary.psync_list_sharerequests(type);

    var shareRequestCount = sharesRaw.deref().sharerequestcnt;

    psync_sharerequest_list_t = StructType({
        sharerequestcnt: 'size_t',
        sharerequests: ArrayType(psync_sharerequest, shareRequestCount)
    });
    initLibrary();

    sharesRaw = MyLibrary.psync_list_sharerequests(type);
    var sharesRequestFromBuf = sharesRaw.deref().sharerequests;

    var listShareRequest = [];

    for (var index = 0; index < shareRequestCount; index++) {
        var shareRequest = {
            sharerequestid: sharesRequestFromBuf[index].sharerequestid,
            folderid: sharesRequestFromBuf[index].folderid,
            created: sharesRequestFromBuf[index].created,
            userid: sharesRequestFromBuf[index].userid,
            toemail: sharesRequestFromBuf[index].toemail,
            sharename: sharesRequestFromBuf[index].sharename,
            message: sharesRequestFromBuf[index].message,
            permissions: sharesRequestFromBuf[index].permissions,
            canread: sharesRequestFromBuf[index].canread,
            cancreate: sharesRequestFromBuf[index].cancreate,
            canmodify: sharesRequestFromBuf[index].canmodify,
            candelete: sharesRequestFromBuf[index].candelete,
            isba: sharesRequestFromBuf[index].isba
        };
        listShareRequest.push(shareRequest);
    }
    return listShareRequest;
}

function getContactsList() {
    var contactsRaw = MyLibrary.psync_list_contacts();

    var contactsCount = contactsRaw.deref().entrycnt;
    // var sharesFromBuf = sharesRaw.deref().shares;
    pcontacts_list = StructType({
        entrycnt: 'uint32',
        entries: ArrayType(contact_info, contactsCount)
    });
    initLibrary();

    contactsRaw = MyLibrary.psync_list_contacts();;
    var contactsFromBuf = contactsRaw.deref().entries;

    var listContacts = [];

    for (var index = 0; index < contactsCount; index++) {
        var contact = {
            teamid: contactsFromBuf[index].teamid,
            mail: contactsFromBuf[index].mail,
            name: contactsFromBuf[index].name,
            type: contactsFromBuf[index].type,
        };

        listContacts.push(contact);
    }
    return listContacts;
}

function getMyTeamsList() {
    var teamsRaw = MyLibrary.psync_list_myteams();

    var teamsCount = teamsRaw.deref().entrycnt;
    // var sharesFromBuf = sharesRaw.deref().shares;
    pcontacts_list = StructType({
        entrycnt: 'uint32',
        entries: ArrayType(contact_info, teamsCount)
    });
    initLibrary();

    teamsRaw = MyLibrary.psync_list_myteams();;
    var contactsFromBuf = teamsRaw.deref().entries;

    var listTeams = [];

    for (var index = 0; index < teamsCount; index++) {
        var team = {
            teamid: contactsFromBuf[index].teamid,
            mail: contactsFromBuf[index].mail,
            name: contactsFromBuf[index].name,
            type: contactsFromBuf[index].type,
        };

        listTeams.push(team);
    }
    return listTeams;
}

function getPublicLinks() {

    let buf = new Buffer(256);
    let ptr = buf.ref();

    let linksRaw = MyLibrary.psync_list_links(ptr);
    let message = ref.deref(ptr).toString('utf-8');
    let terminatingNullPos = message.indexOf('\u0000');

    if (terminatingNullPos >= 0) {
        message = message.substr(0, terminatingNullPos);
    }

    let publicLinksCount = linksRaw.deref().entrycnt;

    plink_info_list_t = StructType({
        entrycnt: 'uint32',
        entries: ArrayType(link_info_t, publicLinksCount)
    });
    initLibrary();

    linksRaw = MyLibrary.psync_list_links(ptr);
    var publicLinksFromBuf = linksRaw.deref().entries;

    var listPublicLinks = [];

    for (var index = 0; index < publicLinksCount; index++) {
        var publicLink = {
            linkid: publicLinksFromBuf[index].linkid,
            name: publicLinksFromBuf[index].name,
            code: publicLinksFromBuf[index].code,
            comment: publicLinksFromBuf[index].comment,
            traffic: publicLinksFromBuf[index].traffic,
            maxspace: publicLinksFromBuf[index].maxspace,
            downloads: publicLinksFromBuf[index].downloads,
            created: publicLinksFromBuf[index].created,
            modified: publicLinksFromBuf[index].modified,
            itemid: publicLinksFromBuf[index].itemid,
            isfolder: publicLinksFromBuf[index].isfolder,
            isupload: publicLinksFromBuf[index].isupload,
            icon: publicLinksFromBuf[index].icon
        };
        listPublicLinks.push(publicLink);
    }
    return listPublicLinks;
}

function openInviteFolder(folderId, folderName, shareList) {
    if (inviteFolderIsOpened()) {
        inviteFolderWindow.webContents.send('folder-info', [folderId, folderName, shareList]);
    } else {
        inviteFolderWindow = new BrowserWindow({
            height: 500,
            width: 600,
            resizable: false,
            modal: true,
            show: false,
            parent: appWindow,
        });
        inviteFolderWindow.setMenu(null);
        //loading the html file
        inviteFolderWindow.loadURL('file://' + __dirname + '/app/invite-to-folder.html');

        inviteFolderWindow.once('ready-to-show', () => {
            // inviteFolderWindow.webContents.openDevTools({ detach: true });
            inviteFolderWindow.show();
            inviteFolderWindow.webContents.send('folder-info', [folderId, folderName, shareList]);
        })
    }
}

function openChooseFolderToShareDialog() {

    let path = dialog.showOpenDialog(appWindow, {
        defaultPath: process.env['HOME'] + '/pCloudDrive/',
        properties: ['openDirectory']
    });

    if (path !== undefined) {
        let pathCrop = path[0].replace(process.env['HOME'] + '/pCloudDrive', '');
        let folderInfoBuffer = MyLibrary.psync_stat_path(pathCrop);

        if (ref.isNull(folderInfoBuffer)) {
            showError('Your path is not valid');
        } else {
            let folderInfoDerefed = folderInfoBuffer.deref();
            let folderUnionFolder = folderInfoBuffer.deref().union.folder;
            let allShares = getShareList(OUTGOING_SHARES);
            appWindow.webContents.send('open-invitation-to-folder', [folderUnionFolder.folderid, folderInfoDerefed.name, allShares, 666]);
        }
    }

}

function deletePublicLink(linkId) {

    let buf = new Buffer(256);
    let ptr = buf.ref();

    let result = MyLibrary.psync_delete_link(linkId, ptr);
    if (result !== 0) {
        let message = ref.deref(ptr).toString('utf-8');
        let terminatingNullPos = message.indexOf('\u0000');
        if (terminatingNullPos >= 0) {
            message = message.substr(0, terminatingNullPos);
            showError(message);
        }
    }
}

function deleteUploadLink(uploadLinkId) {

    let buf = new Buffer(256);
    let ptr = buf.ref();

    let result = MyLibrary.psync_delete_upload_link(uploadLinkId, ptr);
    if (result !== 0) {
        let message = ref.deref(ptr).toString('utf-8');
        let terminatingNullPos = message.indexOf('\u0000');
        if (terminatingNullPos >= 0) {
            message = message.substr(0, terminatingNullPos);
            showError(message);
        }
    }
}

function acceptShareRequest(requestid) {
    let buf = new Buffer(256);
    let ptr = buf.ref();

    let folderId = folderPickerToFolderId('accept-share');
    if (folderId !== undefined) {
        let result = MyLibrary.psync_accept_share_request(requestid, folderId, null, ptr)

        if (result !== 0) {
            let message = ref.deref(ptr).toString('utf-8');
            let terminatingNullPos = message.indexOf('\u0000');
            if (terminatingNullPos >= 0) {
                message = message.substr(0, terminatingNullPos);
                showError(message);
            }
        } else {
            return 1;
        }
    } else {
        return -1;
    }
}

function pathPicker(type) {
    let paths = null;
    let arrayOfPaths = [];
    let initialPath = process.env['HOME'] + '/pCloudDrive';

    if (type === 'files') {
        paths = dialog.showOpenDialog(appWindow, {
            defaultPath: process.env['HOME'] + '/pCloudDrive/',
            properties: ['openFile', 'multiSelections']
        });
    } else {
        paths = dialog.showOpenDialog(appWindow, {
            defaultPath: process.env['HOME'] + '/pCloudDrive/',
            properties: ['openDirectory', 'multiSelections']
        });
    }

    if (paths != undefined && paths[0].startsWith(initialPath)) {

        paths.forEach(function (path) {
            arrayOfPaths.push(path.replace(process.env['HOME'] + '/pCloudDrive', ''));
        });
        return arrayOfPaths;
    } else if (paths != undefined) {
        showError('Only pCloud drive content is allowed!');
        return arrayOfPaths;
    } else {
        return arrayOfPaths;
    }

}

function folderPickerToFolderId(type) {
    let path = null;
    if (type === 'upload') {
        path = dialog.showOpenDialog(appWindow, {
            defaultPath: process.env['HOME'] + '/pCloudDrive',
            properties: ['openDirectory']
        });
    } else {
        path = dialog.showOpenDialog(appWindow, {
            defaultPath: process.env['HOME'] + '/pCloudDrive/Shared',
            properties: ['openDirectory']
        });
    }

    let pathCrop = path[0].replace(process.env['HOME'] + '/pCloudDrive', '');

    if (pathCrop === '') {
        return 0;
    }

    let folderInfoBuffer = MyLibrary.psync_stat_path(pathCrop);
    if (ref.isNull(folderInfoBuffer)) {
        showError('Your path is not valid');
        return;
    } else {
        let folderInfoDerefed = folderInfoBuffer.deref();
        let folderUnionFolder = folderInfoBuffer.deref().union.folder;
        return folderUnionFolder.folderid;
    }

}

function generateUploadLinkFromPicker() {
    let folderId = folderPickerToFolderId('upload');
    generateUploadToFolderLink(folderId);
}

function generateDownloadFileLinkFromPicker() {
    let pickedFiles = pathPicker('files');

    if (pickedFiles.length === 1) {
        //Create single file download link
        generateSingleFileDownloadLink(pickedFiles[0]);
    } else if (pickedFiles.length > 1) {
        // Create virtual folder with the paths
        generateVirtualFolderNamePicker(pickedFiles, 'files');

    }
}

function generateDownloadFolderLinkFromPicker() {
    let pickedFolders = pathPicker('folders');

    if (pickedFolders.length === 1) {
        //Create single folder download link
        generateFolderPublicLink(null, pickedFolders[0]);
    } else if (pickedFolders.length > 1) {
        // Create virtual folder with the paths
        generateVirtualFolderNamePicker(pickedFolders, 'folders');

    }
}

function generateVirtualFolder(vFName, elementsType, arrayPaths) {

    let filesArrayPaths = [];
    let foldersArrayPaths = [];
    let filesArrayCount = 0;
    let foldersArrayCount = 0;
    let buf1 = new Buffer(256);
    let buf2 = new Buffer(256);
    let codePointer = buf1.ref();
    let errPointer = buf2.ref();

    if (elementsType === 'files') {
        filesArrayPaths = arrayPaths;
        filesArrayCount = arrayPaths.length;
    } else {
        foldersArrayPaths = arrayPaths;
        foldersArrayCount = arrayPaths.length;
    }
    // char **folders, int numfolders, char **files, int numfiles

    let result = MyLibrary.psync_tree_public_link(vFName, "", foldersArrayPaths, foldersArrayCount, filesArrayPaths, filesArrayCount, codePointer, errPointer)

    if (result >= 0) {
        let linkCode = ref.deref(codePointer).toString('utf-8');
        let terminatingNullPos = linkCode.indexOf('\u0000');
        if (terminatingNullPos >= 0) {
            linkCode = linkCode.substr(0, terminatingNullPos);
        }
        clipboard.writeText(LINK_INITIAL_ADDRESS_DOWNLOAD + linkCode);
        eNotify.notify({ title: 'Virtual Folder Created', text: 'Link copied to clipboard!', url: LINK_INITIAL_ADDRESS_DOWNLOAD + linkCode });
        virtualFolderWindow.close();
        setTimeout(function () {
            appWindow.webContents.send('reload-shareRoute-page');
        }, 500);

    } else {
        virtualFolderWindow.close();
        let message = ref.deref(errPointer).toString('utf-8');
        let terminatingNullPos = message.indexOf('\u0000');
        if (terminatingNullPos >= 0) {
            message = message.substr(0, terminatingNullPos);
        }
        showError(message);
    }
}

ipcMain.on("close-vf-window", function () {
    virtualFolderWindow.close();
})


// For Outgoing shares
function declineShareRequest(requestid) {

    var choice = require('electron').dialog.showMessageBox(appWindow,
        {
            type: 'question',
            buttons: ['Yes', 'No'],
            title: 'Alert',
            message: 'You will decline the invitation to this folder! Are you sure?'
        });
    if (choice === 0) {
        let buf = new Buffer(256);
        let ptr = buf.ref();

        let result = MyLibrary.psync_decline_share_request(requestid, ptr);
        if (result !== 0) {
            let message = ref.deref(ptr).toString('utf-8');
            let terminatingNullPos = message.indexOf('\u0000');
            if (terminatingNullPos >= 0) {
                message = message.substr(0, terminatingNullPos);
                showError(message);
            }
        }
    }
}

// For Outgoing shares
function declineShareRequestOutgoing(requestid) {

    let buf = new Buffer(256);
    let ptr = buf.ref();

    let result = MyLibrary.psync_cancel_share_request(requestid, ptr);
    if (result !== 0) {
        let message = ref.deref(ptr).toString('utf-8');
        let terminatingNullPos = message.indexOf('\u0000');
        if (terminatingNullPos >= 0) {
            message = message.substr(0, terminatingNullPos);
            showError(message);
        }
    }
}

function removeShare(isBusiness, shareId) {

    let buf = new Buffer(256);
    let ptr = buf.ref();

    let result = 0;

    if (isBusiness === 0) {
        result = MyLibrary.psync_remove_share(shareId, ptr);
    } else {
        result = MyLibrary.psync_account_stopshare(shareId, ptr);
    }
    if (result !== 0) {
        let message = ref.deref(ptr).toString('utf-8');
        let terminatingNullPos = message.indexOf('\u0000');
        if (terminatingNullPos >= 0) {
            message = message.substr(0, terminatingNullPos);
            showError(message);
        }
    }
}

ipcMain.on('close-invite-folder-window', function () {
    inviteFolderWindow.close();
});

function generateVirtualFolderNamePicker(arrayToDisplay, type) {

    virtualFolderWindow = new BrowserWindow({
        height: 400,
        width: 350,
        resizable: false,
        modal: true,
        show: false,
        parent: appWindow,
    });
    virtualFolderWindow.setMenu(null);
    //loading the html file
    virtualFolderWindow.loadURL('file://' + __dirname + '/app/vF.html');

    virtualFolderWindow.once('ready-to-show', () => {
        // virtualFolderWindow.webContents.openDevTools({ detach: true });
        virtualFolderWindow.show();
        virtualFolderWindow.webContents.send('virtual-folder-info', [arrayToDisplay, type]);
    })
}

function generateFolderPublicLink(folderId = null, path = null) {
    let pathCrop = null;
    let buf1 = new Buffer(256);
    let buf2 = new Buffer(256);

    let codePointer = buf1.ref();
    let errPointer = buf2.ref();
    if (folderId != null) {
        path = MyLibrary.psync_fs_get_path_by_folderid(folderId);
        pathCrop = path.replace(process.env['HOME'] + '/pCloudDrive', '');
    } else {
        pathCrop = path;
    }

    let result = MyLibrary.psync_folder_public_link(pathCrop, codePointer, errPointer);

    if (result > 0) {
        let linkCode = ref.deref(codePointer).toString('utf-8');
        let terminatingNullPos = linkCode.indexOf('\u0000');
        if (terminatingNullPos >= 0) {
            linkCode = linkCode.substr(0, terminatingNullPos);
        }
        clipboard.writeText(LINK_INITIAL_ADDRESS_DOWNLOAD + linkCode);
        eNotify.notify({ title: 'Download link generated', text: 'Link copied to clipboard!', url: LINK_INITIAL_ADDRESS_DOWNLOAD + linkCode });
        setTimeout(function () {
            appWindow.webContents.send('reload-shareRoute-page');
        }, 500);

    } else {
        let message = ref.deref(errPointer).toString('utf-8');
        let terminatingNullPos = message.indexOf('\u0000');
        if (terminatingNullPos >= 0) {
            message = message.substr(0, terminatingNullPos);
        }
        showError(message);
    }
}

function generateUploadToFolderLink(folderId) {
    let buf1 = new Buffer(256);
    let buf2 = new Buffer(256);

    let codePointer = buf1.ref();
    let errPointer = buf2.ref();

    let path = MyLibrary.psync_fs_get_path_by_folderid(folderId);
    let pathCrop = path.replace(process.env['HOME'] + '/pCloudDrive', '');

    let result = MyLibrary.psync_upload_link(pathCrop, "Upload Link Generated", codePointer, errPointer);

    if (result > 0) {
        let linkCode = ref.deref(codePointer).toString('utf-8');
        let terminatingNullPos = linkCode.indexOf('\u0000');
        if (terminatingNullPos >= 0) {
            linkCode = linkCode.substr(0, terminatingNullPos);
        }
        clipboard.writeText(LINK_INITIAL_ADDRESS_UPLOAD + linkCode);
        eNotify.notify({ title: 'Upload link generated', text: 'Link copied to clipboard!', url: LINK_INITIAL_ADDRESS_UPLOAD + linkCode });

        setTimeout(function () {
            appWindow.webContents.send('reload-shareRoute-page');
        }, 500);

    } else {
        let message = ref.deref(errPointer).toString('utf-8');
        let terminatingNullPos = message.indexOf('\u0000');
        if (terminatingNullPos >= 0) {
            message = message.substr(0, terminatingNullPos);
        }
        showError(message);
    }
}

function generateSingleFileDownloadLink(path) {
    let buf1 = new Buffer(256);
    let buf2 = new Buffer(256);

    let codePointer = buf1.ref();
    let errPointer = buf2.ref();

    let result = MyLibrary.psync_file_public_link(path, codePointer, errPointer);

    if (result >= 0) {
        let linkCode = ref.deref(codePointer).toString('utf-8');
        let terminatingNullPos = linkCode.indexOf('\u0000');
        if (terminatingNullPos >= 0) {
            linkCode = linkCode.substr(0, terminatingNullPos);
        }
        clipboard.writeText(LINK_INITIAL_ADDRESS_DOWNLOAD + linkCode);
        eNotify.notify({ title: 'Download link generated', text: 'Link copied to clipboard!', url: LINK_INITIAL_ADDRESS_DOWNLOAD + linkCode });
        setTimeout(function () {
            appWindow.webContents.send('reload-shareRoute-page');
        }, 500);

    } else {
        let message = ref.deref(errPointer).toString('utf-8');
        let terminatingNullPos = message.indexOf('\u0000');
        if (terminatingNullPos >= 0) {
            message = message.substr(0, terminatingNullPos);
        }
        showError(message);
    }
}

function modifyShare(share) {
    let buf1 = new Buffer(256);
    let errPointer = buf1.ref();
    let permission = sharePermissionConverter(share.currentPermission);

    let result = MyLibrary.psync_modify_share(share.shareid, permission, errPointer);

    if (result !== 0) {
        let message = ref.deref(errPointer).toString('utf-8');
        let terminatingNullPos = message.indexOf('\u0000');
        if (terminatingNullPos >= 0) {
            message = message.substr(0, terminatingNullPos);
        }
        showError(message);
    } else {
        return result;
    }
}

function shareFolder(folderId, folderName, email, message, permission) {

    let buf1 = new Buffer(256);
    let errPointer = buf1.ref();
    let result = MyLibrary.psync_share_folder(folderId, folderName, email, message, permission, errPointer);

    if (result !== 0) {
        let message = ref.deref(errPointer).toString('utf-8');
        let terminatingNullPos = message.indexOf('\u0000');
        if (terminatingNullPos >= 0) {
            message = message.substr(0, terminatingNullPos);
        }
        showError(message);
        return result;
    } else {
        return result;
    }
}

function accountTeamShare(folderId, folderName, teamId, message, permission) {
    let buf1 = new Buffer(256);
    let errPointer = buf1.ref();
    let result = MyLibrary.psync_account_teamshare(folderId, folderName, teamId, message, permission, errPointer);

    if (result !== 0) {
        let message = ref.deref(errPointer).toString('utf-8');
        let terminatingNullPos = message.indexOf('\u0000');
        if (terminatingNullPos >= 0) {
            message = message.substr(0, terminatingNullPos);
        }
        showError(message);
        return result;
    } else {
        return result;
    }
}

function sharePermissionConverter(permission) {

    if (permission === 0) {
        return 1
    } else if (permission === 1) {
        return 15;
    } else {
        return 16;
    }
}

function loadTextToClipboard(text) {
    clipboard.writeText(text);
    eNotify.notify({ title: 'Link already generated', text: 'Link copied to clipboard!', url: text });
}

function setNotificationOptions() {
    eNotify.setConfig({
        appIcon: path.join(__dirname, 'app/img/notify42.png'),
        width: 320,
        height: 75,
        animateInParallel: true,
        defaultStyleContainer: {
            backgroundColor: '#e6ffff',
            cursor: 'pointer',
            overflow: 'hidden',
            padding: 10,
            fontFamily: 'Arial',
            fontSize: 14,
            position: 'relative',
            lineHeight: '18px',
            right: 0,
            bottom: 0
        },

        defaultStyleAppIcon: {
            overflow: 'hidden',
            float: 'left',
            height: 42,
            width: 42,
            marginRight: 18,
        },
        defaultStyleText: {
            margin: 0,
            fontSize: 13,
            overflow: 'hidden'
        },

        defaultStyleClose: {
            position: 'absolute',
            top: 5,
            right: 5,
            fontSize: 16,
            color: '#000000'
        },
        defaultWindow: {
            alwaysOnTop: true,
            skipTaskbar: true,
            resizable: false,
            show: false,
            frame: false,
            focusable: false,
            backgroundColor: '#e6ffff',
            webPreferences: {
                preload: path.join(__dirname + '/node_modules/electron-notify/', 'preload.js'),
            }
        },
    });
}

function getPathByFolderId(folderId) {
    let path = MyLibrary.psync_fs_get_path_by_folderid(folderId);
    return path;
}

function getPathByFileId(fileId) {
    let buf = ref.alloc('uint64');
    let address = ref.address(buf);
    let path = MyLibrary.psync_get_path_by_fileid(fileId, address);
    return path;
}

function openDestination(id, type) {
    if (type === 'folder') {
        let path = getPathByFolderId(id);
        shell.showItemInFolder(path + "/.");
    } else {
        let fullFilePath = getPathByFileId(id);
        let mountpoint = MyLibrary.psync_fs_getmountpoint();
        shell.showItemInFolder(mountpoint + fullFilePath);
    }
}

function openExternalLocation(url) {
    shell.openExternal(url);
}

function stopShareConfirmation(shareTabName, account = null) {
    let message = "";

    if (shareTabName === "my-shares") {
        message = 'Other people will no longer have access to this folder. Are you sure?';
    } else if (shareTabName === "invite-folder") {
        message = account + ' will no longer have access to this folder. Are you sure?';
    } else {
        message = 'You will no longer have access to this folder! Are you sure?';
    }

    var choice = require('electron').dialog.showMessageBox(appWindow,
        {
            type: 'question',
            buttons: ['Yes', 'No'],
            title: 'Confirm',
            message: message
        });
    if (choice == 1) {
        return false;
    }
    else {
        return true;
    }
}

function inviteFolderIsOpened() {
    let result = BrowserWindow.getAllWindows();
    let inviteToFolderWindowOpened = false;

    for (var x = 0; x < result.length; x++) {
        let window = result[x];
        if (window.getTitle() === "Invite to folder") {
            inviteToFolderWindowOpened = true;
            break;
        }
    }
    return inviteToFolderWindowOpened;
}

function findActiveWindow() {
    let allWindows = BrowserWindow.getAllWindows();

    for (var x = 0; x < allWindows.length; x++) {
        let window = allWindows[x];
        if (window.getTitle() === "Invite to folder") {
            return window;
        } if (window.getTitle() === "Change Password") {
            return window;
        } if (window.getTitle() === "pCloud Login") {
            return window;
        } if (window.getTitle() === "pCloud Register") {
            return window;
        } if (window.getTitle() === "Provide email") {
            return window;
        } if (window.getTitle() === "Add New Sync") {
            return window;
        } if (window.getTitle() === "Create Download Link") {
            return window;
        } if (window.getTitle() === "Unlock Crypto Folder") {
            return window;
        } if (window.getTitle() === "Confirmation") {
            return window;
        } if (window.getTitle() === "pCloud Drive") {
            return window;
        }
    }
    return null;
}

function setCurrentOpenTab(tab) {
    currentOpenTab = tab;
}
